; $Id: dcs_cryst_align.pro,v 1.41.2.2 2013/05/29 15:09:24 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dcs_cryst_align
;
; PURPOSE:
;  This program plots crystal alignment files
;
; CATEGORY:
;  DCS data tools
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  May, 2013
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

@dm_check_macskidney

pro dcs_cryst_align_e2lambda_event,event
    widget_control,/hourglass
    widget_control,event.handler,get_uvalue=state

    ;catch and ignore all errors in this program
    catch, myerror
    if myerror ne 0 then begin
       ok = dialog_message(dialog_parent=state.tlb,!error_state.msg,/error,/center)
       catch,/cancel
       return
    end

    case event.id of
       state.eTxt:     dcs_cryst_align_e2lambda_update,state,/E
       state.lTxt:     dcs_cryst_align_e2lambda_update,state,/lambda
       state.plotBut:  DCSToolsPerformance,group_leader=event.top
       state.exitBut:  begin
                       widget_control,event.handler,/destroy
                       return
                       end
       else:
    endcase
 
    widget_control,event.handler,set_uvalue=state
end

pro dcs_cryst_align_e2lambda_update,state,E=E,lambda=lambda
    eng = !values.f_nan & wav = !values.f_nan   ;initialization
    ;retrieve values from text boxes
    if keyword_set(E) then widget_control,state.eTxt,get_value=str 
    if keyword_set(lambda) then widget_control,state.lTxt,get_value=str
    if n_elements(str) ne 0 then begin
       tmp = dm_to_number(strsplit(str,' ,'+string(9b),/extract))
       ind0 = where(finite(tmp),count0)
       if count0 gt 0 then begin
          ind1 = where(tmp[ind0] gt 0,count1)
          if count1 gt 0 then begin
             if keyword_set(E) then begin
                eng = tmp[ind0[ind1]]
                wav = sqrt(81.8042/eng)
                widget_control,state.lTxt,set_value=dm_to_string(wav,resolution=3,separator=', ')
             endif
             if keyword_set(lambda) then begin
                wav = tmp[ind0[ind1]]
                eng = 81.8042/wav/wav
                widget_control,state.eTxt,set_value=dm_to_string(eng,resolution=3,separator=', ')
             endif
          endif
       endif
    endif
    ;find out the energy resolution and flux 
    wav = wav[0] & eng = eng[0] ;only the first one  
    if finite(eng) and finite(wav) then begin
       f_low  = interpol((*(state.flPtr))[1,*],(*(state.flPtr))[0,*],wav)/state.fmax*100.0
       f_med  = interpol((*(state.fmPtr))[1,*],(*(state.fmPtr))[0,*],wav)/state.fmax*100.0
       flux   = 'low:'+strtrim(string(0>(f_low)<100,format='(f6.2)'),2)+', '+$
                'med:'+strtrim(string(0>(f_med)<100,format='(f6.2)'),2)
       if state.isdcs then widget_control,state.fxTxt,set_value=flux
       er_low = 13220./wav^2.947
       er_med = 3948./wav^2.656
       eresl  = 'low:'+strtrim(string(er_low,format='(f9.1)'),2)+', '+$
                'med:'+strtrim(string(er_med,format='(f9.1)'),2)
       if state.isdcs then widget_control,state.erTxt,set_value=eresl
    endif else begin
       if state.isdcs then widget_control,state.fxTxt,set_value=''
       if state.isdcs then widget_control,state.erTxt,set_value=''
       if keyword_set(E) then      widget_control,state.lTxt,set_value=''
       if keyword_set(lambda) then widget_control,state.eTxt,set_value=''
    endelse
end

pro dcs_cryst_align_e2lambda_Exit,tlb
    widget_control,tlb,get_uvalue=state
    ptr_free,state.flPtr,state.fmPtr
    widget_control,tlb,/destroy
end

;E to Lambda conversion
pro dcs_cryst_align_e2lambda,event,lambda=lambda,dcs=dcs
    state={group_leader:    0L, $   ;group leader
           tlb:             0L, $   ;top level base
           isdcs:           0b, $   ;flag for DCS instrument
           eTxt:            0L, $   ;E text box
           lTxt:            0L, $   ;lambda text box
           erTxt:           0L, $   ;E resolution text box
           fxTxt:           0L, $   ;flux text box
           plotBut:         0L, $   ;start performance button
           exitBut:         0L, $   ;exit button
           fmax:            0e, $   ;maximum flux
           flPtr:    ptr_new(), $   ;low resl mode flux
           fmPtr:    ptr_new()  $   ;medium resl mode flux
          }
    registerName = 'dcs_cryst_align_e2lambda'
    if xregistered(registerName) then begin ;only allow one copy running
       id = LookupManagedWidget(registername)
       widget_control,id,iconify=0
       return
    endif
    state.isdcs = keyword_set(dcs)
    if n_elements(lambda) eq 0 then lambda=!values.f_nan
    if n_elements(event) ne 0 then begin
       state.group_leader = event.top
       state.tlb = widget_base(title='Convert Energy to Wavelength',/col,kill_notify='dcs_cryst_align_e2lambda_Exit',tlb_frame_attr=1,group_leader=event.top)
    endif else $
       state.tlb = widget_base(title='Convert E to Wavelength',/col,kill_notify='dcs_cryst_align_e2lambda_Exit',tlb_frame_attr=1)
    row1        = widget_base(state.tlb,/row,/grid_layout)
    col1        = widget_base(row1,/col,/grid_layout)
    col2        = widget_base(row1,/col,/grid_layout)
    void        = widget_label(col1,value='E (meV)',/align_center)
    state.eTxt  = widget_text(col1,/editable,/all_events)
    void        = widget_label(col2,value='Lambda ('+string('c5'xb)+')',/align_center)
    state.lTxt  = widget_text(col2,/editable,/all_events,value=dm_to_string(lambda))
    if state.isdcs then begin
    void        = widget_label(col1,value='FWHM ('+string('b5'xb)+'eV)',/align_center)
    state.erTxt = widget_text(col1)
    void        = widget_label(col2,value='Flux (% of maximum)',/align_center)
    state.fxTxt = widget_text(col2)
    endif
    row2        = widget_base(state.tlb,/row,/align_center)
    if state.isdcs then begin
    state.plotBut = widget_button(row2,value='  Plot  ',sensitive=(xregistered('dave',/noshow) or xregistered('wd_DAVETool',/noshow)))
    void          = widget_label(row2,value= '   ')
    endif
    state.exitBut = widget_button(row2,value='  Exit  ')
    ;flux info
    state.flPtr = ptr_new([[1.25,357], [1.3,1799.4], [1.35,30630], [1.36,44460], [1.38,69780], [1.4,86595], [1.42,86820], [1.44,59370], [1.45,57570], [1.46,73200],$
                    [1.48,117900], [1.5,137498], [1.52,143280], [1.54,145380], [1.55,135750], [1.56,106530], [1.58,33600], [1.6,34440], [1.62,122010], [1.64,177630],$
                    [1.65,172500], [1.66,158280], [1.68,167760], [1.7,212850], [1.72,165270], [1.74,68310], [1.75,51450], [1.76,71100], [1.78,186870], [1.8,266385],$
                    [1.82,265980], [1.84,216420], [1.85,177360], [1.86,131010], [1.88,43860], [1.9,4959], [1.95,3192], [1.96,7602], [1.98,32730], [2,87915],$
                    [2.02,116220], [2.04,60900], [2.05,28993.5], [2.06,10188], [2.08,820.8], [2.1,167.22], [2.15,385.5], [2.2,196260], [2.25,541500], [2.3,720400],$
                    [2.4,803100], [2.5,875400], [2.6,905700], [2.7,912600], [2.8,902400], [2.9,898500], [3,901350], [3.1,888300], [3.2,876600], [3.25,855900],$
                    [3.27,826800], [3.3,760800], [3.33,390900], [3.35,346200], [3.38,731100], [3.4,831900], [3.42,839400], [3.45,846000], [3.5,861300], [3.6,870300],$
                    [3.7,877800], [3.8,859500], [3.85,844500], [3.9,832200], [3.95,807000], [3.98,781200], [4,763800], [4.03,734700], [4.05,787800], [4.08,850500],$
                    [4.1,872400], [4.15,857400], [4.2,847200], [4.3,828900], [4.4,798300], [4.5,765900], [4.6,738000], [4.62,719400], [4.65,716400], [4.67,746400],$
                    [4.7,780000], [4.72,777900], [4.75,772800], [4.78,766200], [4.8,763950], [5,721650], [5.5,645900], [6,558000], [6.3,508200], [6.4,487500],$
                    [6.45,466500], [6.5,453600], [6.55,405600], [6.6,297180], [6.65,33900], [6.7,24780], [6.75,279690], [6.8,384600], [6.85,402600], [6.9,413700],$
                    [6.95,406200], [7,404850], [7.5,349800], [8,292040], [9,199770], [10,142250], [12,69260], [14,34890], [15,25104], [16,17742],$
                    [18,9459], [20,5184]])
    state.fmPtr = ptr_new([[1.35,3894], [1.4,11083.5], [1.45,4354.5], [1.5,17070], [1.55,17355], [1.6,1409.55], [1.65,22560], [1.7,27450], [1.75,3330], [1.8,34275],$
                    [1.85,22425], [1.9,207.45], [1.95,322.2], [2,11304], [2.05,1801.5], [2.1,130.185], [2.15,150.9], [2.2,28005], [2.25,72900], [2.3,92160],$
                    [2.4,102165], [2.5,112905], [2.6,117765], [2.7,120540], [2.8,117645], [2.9,119475], [3,122535], [3.2,117090], [3.25,114390], [3.3,103335],$
                    [3.35,31950], [3.4,112305], [3.5,116445], [3.6,120810], [3.7,120810], [3.8,119025], [3.85,118320], [3.9,115035], [3.95,111540], [4,105735],$
                    [4.05,106050], [4.1,123960], [4.15,121905], [4.2,121185], [4.3,117795], [4.4,113085], [4.6,103245], [4.8,110460], [5,103703], [5.5,91095],$
                    [6,77842.5], [6.5,59820], [6.6,36780], [6.7,952.5], [6.8,49335], [6.9,53190], [7,54127.5], [7.5,43965], [8,36540], [9,24315],$
                    [10,16657.5], [12,7679.25], [14,3831], [16,1926], [18,1006.58], [20,554.55]])
    state.fmax = max((*(state.flPtr))[1,*])
    dm_center_kid,state.tlb,state.group_leader
    widget_control,state.tlb,/realize
    dcs_cryst_align_e2lambda_update,state,/lambda
    widget_control,state.tlb,set_uvalue=state
    xmanager,registerName,state.tlb,cleanup='dcs_cryst_align_e2lambda_Exit',/no_block
end

pro dcs_cryst_align_kidlimit_event,event
    widget_control,/hourglass
    widget_control,event.handler,get_uvalue=state

    ;catch and ignore all errors in this program
    catch, myerror
    if myerror ne 0 then begin
       ok = dialog_message(dialog_parent=state.tlb,!error_state.msg,/error,/center)
       catch,/cancel
       return
    end

    case event.id of
       state.eTxt:     begin
                       widget_control,event.id,get_value=str
                       Ei = dm_to_number(strsplit(str,' ,'+string(9b),/extract))
                       ind0 = where(finite(Ei),count0)
                       if count0 gt 0 then begin
                          ind1 = where(Ei[ind0] gt 0,count1)
                          if count1 gt 0 then begin
                             llim = interpol((*state.kidlimit)[*,1],(*state.kidlimit)[*,0],Ei[ind0[ind1]])
                             ulim = interpol((*state.kidlimit)[*,2],(*state.kidlimit)[*,0],Ei[ind0[ind1]])
                             widget_control,state.lTxt,set_value=dm_to_string(llim-state.offset,resolution=3,separator=', ')
                             widget_control,state.uTxt,set_value=dm_to_string(ulim-state.offset,resolution=3,separator=', ')
                             valid = 1b
                          endif
                       endif 
                       if ~keyword_set(valid) then begin
                          widget_control,state.lTxt,set_value=''
                          widget_control,state.uTxt,set_value=''
                       endif
                       end
       state.oTxt:     begin
                       old_offset = state.offset
                       widget_control,event.id,get_value=str
                       state.offset = dm_to_number(str)
                       if finite(state.offset,/nan) then state.offset = 0.0
                       widget_control,event.handler,set_uvalue=state
                       if state.offset ne old_offset then begin
                          if obj_valid(state.plotobj) then begin
                             event.id = state.plotBut
                             dcs_cryst_align_kidlimit_event,event
                          endif
                          event.id = state.eTxt
                          dcs_cryst_align_kidlimit_event,event
                          widget_control,state.oTxt,/input_focus
                       endif
                       end
       state.plotBut:  begin
                       if ~obj_valid(state.plotobj) then state.plotobj = obj_new('dm_plot',legdfsize=10,background=state.bgcolor)
                       state.plotobj->setproperty,xdat=(*state.kidlimit)[*,0],ydat=(*state.kidlimit)[*,1]-state.offset,/no_copy,legend='lower limit',color='green',xran=[2,17],yran=[-60,55],$
                            psym='filled circle',xtit='E!di!n (meV)',ytit='Kidney Limits (\deg)',linestyle='solid',/nodraw,hidelegend=0,legdpos=[0.6,0.9],/legdshowoutline,/showcursorpos
                       state.plotobj->add_plot,(*state.kidlimit)[*,0],(*state.kidlimit)[*,2]-state.offset,legend='upper limit',color='red',psym='filled circle',linestyle='solid'
                       state.plotobj->draw
                       end
       state.exitBut:  begin
                       widget_control,event.handler,/destroy
                       return
                       end
       else:
    endcase
    
    widget_control,event.handler,set_uvalue=state
end

pro dcs_cryst_align_kidlimit_Exit,tlb
    widget_control,tlb,get_uvalue=state
    ptr_free,state.kidlimit
    if obj_valid(state.plotobj) then obj_destroy,state.plotobj
    widget_control,tlb,/destroy
end

;macs kidney limit
pro dcs_cryst_align_kidlimit,event,bgcolor=bgcolor
    state={group_leader:    0L, $   ;group leader
           tlb:             0L, $   ;top level base
           eTxt:            0L, $   ;E text box, editable
           oTxt:            0L, $   ;offset angle text box
           offset:          0e, $   ;offset angle value
           lTxt:            0L, $   ;lower limit text box
           uTxt:            0L, $   ;upper limit text box
           plotBut:         0L, $   ;plot limits button
           exitBut:         0L, $   ;exit button
           kidlimit: ptr_new(), $   ;kidney limits
           bgcolor:         '', $   ;plot window background color
           plotobj:  obj_new()  $   ;dm_plot
          }
    registerName = 'dcs_cryst_align_kidlimit'
    if xregistered(registerName) then begin ;only allow one copy running
       id = LookupManagedWidget(registername)
       widget_control,id,iconify=0
       return
    endif
    
    if n_elements(bgcolor) ne 0 then state.bgcolor = bgcolor[0]
    
    tmp = dm_check_macskidney(0,5,limits=limits,offset=offset)
    state.kidlimit = ptr_new(limits)
    state.offset   = offset
    
    if n_elements(event) ne 0 then begin
       state.group_leader = event.top
       state.tlb  = widget_base(title='Calculate Kidney Angle Limits',/col,kill_notify='dcs_cryst_align_kidlimit_Exit',tlb_frame_attr=1,group_leader=event.top)
    endif else $
       state.tlb  = widget_base(title='Calculate Kidney Angle Limits',/col,kill_notify='dcs_cryst_align_kidlimit_Exit',tlb_frame_attr=1)
    row1          = widget_base(state.tlb,/row,/grid_layout)
    col1          = widget_base(row1,/col,/grid_layout)
    col2          = widget_base(row1,/col,/grid_layout)
    col3          = widget_base(row1,/col,/grid_layout)
    col4          = widget_base(row1,/col,/grid_layout)
    void          = widget_label(col1,value='Ei (meV)',/align_center)
    state.eTxt    = widget_text(col1,/editable,/all_events)
    void          = widget_label(col2,value='Lower Limit ('+string('b0'XB)+')',/align_center)
    state.lTxt    = widget_text(col2)
    void          = widget_label(col3,value='Upper Limit ('+string('b0'XB)+')',/align_center)
    state.uTxt    = widget_text(col3)
    void          = widget_label(col4,value='Offset Angle ('+string('b0'XB)+')',/align_center)
    state.oTxt    = widget_text(col4,/editable,/all_events,value=dm_to_string(state.offset,resolution=3))
    row2          = widget_base(state.tlb,/row,/align_center) 
    void          = widget_label(row2,value='The limits are software angles using the offset angle.')
    row3          = widget_base(state.tlb,/row,/align_center) 
    state.plotBut = widget_button(row3,value='  Plot  ')
    void          = widget_label(row3,value= '   ')
    state.exitBut = widget_button(row3,value='  Exit  ')
    dm_center_kid,state.tlb,state.group_leader
    widget_control,state.tlb,/realize
    widget_control,state.tlb,set_uvalue=state
    xmanager,registerName,state.tlb,cleanup='dcs_cryst_align_kidlimit_Exit',/no_block
end

;plotting
pro dcs_cryst_align_plot,pState,dataonly=dataonly,xs=xs,ys=ys,dys=dys
    if (*pState).fit then (*pState).plotWin->getproperty,xran=oldxran ;save the xrange
    if n_elements(*(*pState).yPtr) eq 0 then return ;no data
    tit    = (*pState).title  
    xtit   = (*(*pState).xtit)[(*pState).xaxis]
    ytit   = 'Neutron Counts'
    n_file = n_elements((*(*pState).xPtr)[0,*]) 
    n_det  = n_elements(*(*pState).detPtr)
    case (*pState).ftype of 
         0:   ytit = ytit+'/'+dm_to_string((*pState).duration,/int)+' Secs'
         2:   begin
              xA4 = stregex(xtit,'A4',/fold_case,/boolean)  ;Is x axis A4?
              yMn = ((*(*pState).detPtr)[0] eq -1)          ;Is y axis monitor?
              iA4 = where(stregex((*(*pState).xtit),'A4',/fold_case,/boolean))
              mon = reform((*(*pState).xPtr)[n_elements((*(*pState).xPtr)[*,0])-1,*])
              titind = stregex(tit,'\(2th=([0-9.]+)',/subexpr,length=titlen)
              if titlen[1] gt 0 then tittth = dm_to_number(strmid(tit,titind[1],titlen[1])) else begin
                 titind = stregex(tit,'\(<2th>=([0-9.]+)',/subexpr,length=titlen)
                 if titlen[1] gt 0 then tittth = dm_to_number(strmid(tit,titind[1],titlen[1]))
              endelse
              if (*pState).duration lt 0 then $
                 ytit = ytit+'/'+dm_to_string(-(*pState).duration,/int)+' Secs' $
              else $
                 ytit = ytit+'/mon='+dm_to_string((*pState).duration,/exp)
              end
         else:
    endcase
    showindvdet = (*pState).indvdet
    if strpos(tit,'det=all') ne -1     then showindvdet = 0b 
    if strpos(tit,'det=upper') ne -1   then showindvdet = 0b 
    if strpos(tit,'det=central') ne -1 then showindvdet = 0b 
    if strpos(tit,'det=lower') ne -1   then showindvdet = 0b 
    if keyword_set(dataonly)           then showindvdet = 0b 
    hidelegend = (n_elements(*(*pState).detPtr) eq 1) or (~showindvdet)
    colrs = (*pState).plotWin->getcolor(/list)
    colrs = colrs[0:n_elements(colrs)-2] ;remove the user define color
    for ii = 0L, 0>(showindvdet*n_det-1) do begin
        x = reform((*(*pState).xPtr)[(*pState).xaxis,*])
        y = fltarr(n_file) & dy = y
        for i=0,n_file-1 do begin
            if showindvdet then begin
               if (*pState).ftype eq 2 then begin
                  if ~yMn then begin
                     y[i]  = (*(*pState).yPtr)[(*pState).y_typ,(*(*pState).detPtr)[ii],i]
                     dy[i] = (*(*pState).dyPtr)[(*pState).y_typ,(*(*pState).detPtr)[ii],i]
                     if xA4 then x[i] = x[i]-76+(*(*pState).tthPtr)[(*(*pState).detPtr)[ii]]
                  endif
               endif else begin
                  y[i]  = (*(*pState).yPtr)[(*(*pState).detPtr)[ii],i]
                  dy[i] = (*(*pState).dyPtr)[(*(*pState).detPtr)[ii],i]
               endelse
            endif else begin
               if (*pState).ftype eq 2 then begin
                  if ~yMn and ~xA4 then begin
                     y[i]  = total((*(*pState).yPtr)[(*pState).y_typ,*(*pState).detPtr,i],/nan)
                     dy[i] = sqrt(total((*(*pState).dyPtr)[(*pState).y_typ,*(*pState).detPtr,i]^2,/nan))
                  endif
               endif else begin
                  y[i]  = total((*(*pState).yPtr)[*(*pState).detPtr,i],/nan)
                  dy[i] = sqrt(total((*(*pState).dyPtr)[*(*pState).detPtr,i]^2,/nan))
               endelse
            endelse
        endfor
        if keyword_set(yMn) then begin
           y  = mon
           dy = sqrt(mon)
           if xA4 then begin
              x  = (fltarr(20)+1)#x
              for i=0,19 do begin
                  x[i,*] = x[i,*]-76+(*(*pState).tthPtr)[i]
              endfor
              y  = (fltarr(20)+1)#y
              dy = (fltarr(20)+1)#dy
           endif
        endif  
        if ~showindvdet and keyword_set(xA4) then begin ;special case of MACS A4
           if ~yMn then begin
              x = (fltarr(n_det)+1)#x
              for i=0,n_det-1 do begin
                  x[i,*] = x[i,*]-76+(*(*pState).tthPtr)[(*(*pState).detPtr)[i]]
              endfor
              x = reform(x)
              y  = reform((*(*pState).yPtr)[(*pState).y_typ,*(*pState).detPtr,*])
              dy = sqrt(y)
           endif
        endif
        tempaxis = ((*pState).xaxis eq (n_elements(*(*pState).xtit)-1))
        if tempaxis and (n_file gt 1) then begin  ;combine data with same T_set
           if (*pState).ftype eq 0 then $
              T_set = reform((*(*pState).xPtr)[9,*]) $
           else $
              T_set = round(x*100.0)/100.0
           tmp = T_set  
           dm_step_bin,0,T_set,ydat=x,uniq_val=tmp  
           dm_step_bin,0,tmp,ydat=y,yerr=dy,uniq_val=tmp
        endif 
        if keyword_set(xA4) then begin  ;combine MACS data with same A4
           uniq_x = round(x*(100d)-(0.501d),/L64)*0.01
           dm_step_bin,0,x,ydat=y,yerr=dy,uniq_val=uniq_x
        endif  
        colr = colrs[ii mod n_elements(colrs)] 
        lsty = (['no line',(['solid','dashed'])[(ii/15) mod 2]])[~hidelegend]
        if showindvdet then begin
           if (*pState).ftype eq 0 then begin
              if (*(*pState).detPtr)[ii] eq 913 or (*(*pState).detPtr)[ii] eq 914 then begin
                 legd = (['FC2','FC3'])[(*(*pState).detPtr)[ii]-913]
              endif else begin
                 legd = dm_to_string((*(*pState).detPtr)[ii]+1)+([' L',' C',' U'])[dm_mask_detbanks(detnum=(*(*pState).detPtr)[ii])+1]
              endelse 
           endif else legd = dm_to_string((*(*pState).detPtr)[ii]+1)
        endif else legd = ''
        if ii eq 0 then begin 
           xs  = x
           ys  = y
           dys = dy
           if ~keyword_set(dataonly) then $
              (*pState).plotWin->setproperty,xdat=x,ydat=y,yerr=dy,/nodraw,title=tit,xtit=xtit,ytit=ytit,psym='circle',hidelegend=hidelegend,linestyle=lsty,color=colr,legend=legd
        endif else begin
           xs  = [xs,x]
           ys  = [ys,y]
           dys = [dys,dy]
           if ~keyword_set(dataonly) then $
              (*pState).plotWin->add_plot,x,y,yerr=dy,psym='circle',linestyle=lsty,color=colr,legend=legd
        endelse
    endfor
    if keyword_set(dataonly) then return
    xmax = max(xs,min=xmin) 
    ymax = max(ys+dys,imax) & imax = imax[0]
    ymin = min(ys-dys)
    xran = [xmin,xmax]+[-0.05,0.05]*(xmax-xmin+abs(xmax+1)/10.)
    yran = [ymin,ymax]+[-0.05,0.05]*(ymax-ymin)
    (*pState).plotWin->remove_text
    index = uniq(x,sort(x))
    if (*pState).fit and (n_elements(index) gt 1) and (n_elements(y) gt 4) and ((~(*pState).indvdet) or (n_elements(*(*pState).detPtr) eq 1)) then begin
       if total(finite(oldxran,/nan)) eq 0 then begin
          index = where((x ge oldxran[0]) and (x le oldxran[1]),count)
          if count gt 4 then begin
             x = x[index]
             y = y[index]
          endif
       endif
       dm_gaussfit,y,x=x,params=params
       if total(finite(params)) eq n_elements(params) then begin
          x0 = xmin+findgen(150)/149*(xmax-xmin+2)-1 
          if params[2] ne 0 then begin
             xx = params[1]-4*params[2]+findgen(60)/59*8*params[2]
             id = where(x0 lt params[1]-4*params[2],count)
             if count gt 0 then xx = [x0[id],xx]
             id = where(x0 gt params[1]+4*params[2],count)
             if count gt 0 then xx = [xx,x0[id]]
             yy = params[0]*exp(-0.5*((xx-params[1])/params[2])^2)+params[3]
          endif else begin
             xx = x0
             yy = params[3]+fltarr(n_elements(x0))
          endelse
          (*pState).plotWin->add_plot,xx,yy,linestyle='solid',color='red',legend='gauss fit',layer=0
          text = 'peak!l !n='+dm_to_string(params[1],resolution=2)+'!C'+$
                 'fwhm='+dm_to_string(params[2]*sqrt(8.*alog(2)),resolution=3)
          if ((*pState).ftype eq 2) and (n_elements(tittth) ne 0) then begin
             tmp = min(abs(x-params[1]),ind)
             if ~xA4 then tittth = mean((*(*pState).xPtr)[iA4,ind[0]])-76.0+tittth else tittth = mean((*(*pState).xPtr)[iA4,*])-76.0+tittth 
             tit = strmid(tit,0,titind[1])+dm_to_string(tittth,res=3)+strmid(tit,titind[1]+titlen[1])
             (*pState).tth[1] = tittth
          endif
          ypos = 0.8
          if ((*pState).xaxis eq 0) and finite((*pState).tth[(*pState).ftype eq 2]) then begin
             tth = (*pState).tth[(*pState).ftype eq 2] 
             if (*pState).ftype eq 1 then tth = -tth  ;WAND
             if tth ge 0 then a2=90.-tth/2. else a2=-(90.+tth/2.)
             if (*pState).ftype eq 2 then a2 = 90-a2  ;MACS
             text = text+'!C'+(*(*pState).a2name)[(*pState).ftype]+' reset to '+dm_to_string(a2,resolution=3)+string('b0'xb)              
             ypos = ypos-0.05
          endif
          if (*pState).showfit then begin
             text = text+'!Cparams=['+dm_to_string(params,sep=',')+']!n' 
             ypos = ypos-0.05
          endif
          (*pState).plotWin->add_text,text,0.05,ypos,fontsize=14
       endif
    endif else begin
       if ((*pState).ftype eq 2) and (n_elements(tittth) ne 0) then begin
          tittth = mean((*(*pState).xPtr)[iA4,*])-76.0+tittth 
          tit = strmid(tit,0,titind[1])+dm_to_string(tittth,res=3)+strmid(tit,titind[1]+titlen[1])
          (*pState).tth[1] = tittth
       endif
    endelse
    (*pState).plotWin->setproperty,xran=xran,yran=yran,/nodraw,title=tit
    (*pState).plotWin->draw
end

;calculate the angle between hkl1 and hkl2
function dcs_cryst_align_angle,hkl1,hkl2,dir_latt_parm
    abc_r = dm_basis_r(dir_latt_parm)
    q1 = abc_r#hkl1
    q2 = abc_r#hkl2
    return,acos(q1##transpose(q2)/norm(q1)/norm(q2))/!dtor
end

;calculate q length from direct crystal lattice parameters
function dcs_cryst_align_qlength,hkl,cryst
    ddtor   = !dpi/(180d)
    sinabg  = sin(cryst[3:5]*ddtor)
    cosabg  = cos(cryst[3:5]*ddtor)
    v       = cryst[0]*cryst[1]*cryst[2]*sqrt((1d)+(2d)*cosabg[0]*cosabg[1]*cosabg[2]-total(cosabg[0:2]^2))
    rec_abc = (2d)*!dpi/v*[cryst[1]*cryst[2]*sinabg[0],cryst[0]*cryst[2]*sinabg[1],cryst[0]*cryst[1]*sinabg[2]]
    tmp     = (4d)*!dpi*!dpi/v/v*cryst[0]*cryst[1]*cryst[2]
    aadotbb = tmp*cryst[2]*(cosabg[0]*cosabg[1]-cosabg[2])
    bbdotcc = tmp*cryst[0]*(cosabg[1]*cosabg[2]-cosabg[0])
    ccdotaa = tmp*cryst[1]*(cosabg[2]*cosabg[0]-cosabg[1])
    return, sqrt(total((hkl*rec_abc)^2)+(2d)*(hkl[0]*hkl[1]*aadotbb+hkl[1]*hkl[2]*bbdotcc+hkl[2]*hkl[0]*ccdotaa))
end

;change file type
pro dcs_cryst_align_changeftype,event,init=init,pState=pState,det_info=det_info,x_info=x_info
    if keyword_set(init) then begin
       *(*pState).fname  = ['DCS','WAND','MACS']
       *(*pState).fextn  = ['.dcs','.wand','.ng0,.bt9']
       *(*pState).a2name = ['A2','Psi','A3']
    endif else begin
       uname = widget_info(event.id,/uname)
       if strmid(uname,0,5) ne 'ftype' then return
       widget_control,event.handler,get_uvalue=pState
       ftype = dm_to_number(strmid(uname,6,strlen(uname)-6),/int) 
       if (*pState).ftype eq ftype then return else (*pState).ftype = ftype
       ftypemenu  = widget_info((*pState).tlb,find_by_uname='ftypeMenu')
       checked    = widget_info(ftypemenu,find_by_uname='ftype_'+dm_to_string((*pState).ftype))
       for i=0,n_elements(*(*pState).fname)-1 do begin
           if i ne (*pState).ftype then begin
              tmp = widget_info(ftypemenu,find_by_uname='ftype_'+dm_to_string(i))
              unchecked = (n_elements(unchecked) eq 0)?tmp:([unchecked,tmp])
           endif
       endfor
       dm_toggle_menubut,check=checked,uncheck=unchecked
       widget_control,widget_info((*pState).tlb,find_by_uname='optnbitflip'),sensitive=((*pState).ftype eq 0)
       widget_control,widget_info((*pState).tlb,find_by_uname='timelabel'),sensitive=((*pState).ftype eq 0)
       widget_control,widget_info((*pState).tlb,find_by_uname='timelist'),sensitive=((*pState).ftype eq 0)
       optnmenu = widget_info((*pState).tlb,find_by_uname='optnMenu')
       toolmenu = widget_info((*pState).tlb,find_by_uname='toolMenu')
       if (*pstate).ftype eq 2 then begin  ;MACS
          macsdetmenu = widget_button(optnmenu,value='MACS Detector Choice',/menu,uname='macsdetMenu',/separator)
          void = dm_widget_button(macsdetmenu,value='SPEC',uname='macs_det_spec',set_button=((*pState).y_typ eq 0))
          void = dm_widget_button(macsdetmenu,value='DIFF',uname='macs_det_diff',set_button=((*pState).y_typ eq 1))
          kidlimitBut = widget_button(toolmenu,value='Kidney Angle Limit',uname='kidlimitbut')
       endif else begin             
          macsdetmenu = widget_info(optnmenu,find_by_uname='macsdetMenu')
          if macsdetmenu ne 0 then widget_control,macsdetmenu,/destroy
          kidlimitBut = widget_info(toolmenu,find_by_uname='kidlimitbut')
          if kidlimitBut ne 0 then widget_control,kidlimitBut,/destroy
       endelse
    endelse
    
    ;initialize the instrument info
    (*pState).t_chan  = 1
    (*pState).e_ran   = [!values.f_nan,!values.f_nan]
    (*pState).tth     = [!values.f_nan,!values.f_nan]
    (*pState).xaxis   = 0
    (*pState).fit     = 0
    case (*pState).ftype of   ;temperature needs to be the last entry of xtit
       0:begin    ;DCS
         alldetPtr    = dm_load_detpos(/dcs)
         det_info     = ['All','Specify','Central Bank','Upper Bank','Lower Bank','FC2','FC3']
         x_info       = ['Rotation','Height','X-translation','Y-translation','X-tilt','Y-tilt','File Index','Temperature']
         *(*pState).xtit = ['Rotation (\deg)','Height (mm)','X-translation (mm)','Y-translation (mm)','X-tilt (\deg)','Y-tilt (\deg)','File Index','T (K)']
         (*pState).tch_tit = 'tchan='+(['all','elastic','specify'])[(*pState).t_chan]
         end
       1:begin    ;WAND
         alldetPtr    = dm_load_detpos(/wand) 
         det_info     = ['All','Specify']
         x_info       = ['Psi','Temperature']
         *(*pState).xtit = ['Psi (\deg)','T (K)']
         (*alldetPtr).two_theta  = (*alldetPtr).two_theta*(-1) 
         (*(*pState).infoPtr)[0] = 1.46 ;fixed wavelength
         (*pState).tch_tit = ''
         end
       2:begin    ;MACS
         alldetPtr    = dm_load_detpos(/macs) 
         det_info     = ['All','Specify','Monitor']
         x_info       = ['A3','A4','A1','A2','A5','A6','MonRot','DFMDTS','MonBlade'+string(1+indgen(21),format='(i02)'),$
                        ['X','Y','Z']+'-translation','Lower-tilt','Upper-tilt','Temperature']
         *(*pState).xtit = [['A3','A4','A1','A2','A5','A6','MonRot']+' (\deg)','DFMDTS (mm)','MonBlade'+string(1+indgen(21),format='(i02)')+' (\deg)',$
                        ['X','Y','Z']+' (mm)',['L','U']+'-tilt (\deg)','T (K)']
         (*pState).tch_tit = ''
         end  
       else:
    endcase
    *(*pState).tthPtr = (*alldetPtr).two_theta & ptr_free,alldetPtr
    (*pState).ndet    = n_elements(*(*pState).tthPtr)
    *(*pState).detPtr = indgen((*pState).ndet)  ;default all detectors
    (*pState).det_tit = (['','',(['spec ','diff '])[(*pState).y_typ]])[(*pState).ftype]+'det=all'
    (*pState).title   = (*pState).det_tit
    if strlen((*pState).tch_tit) ne 0 then (*pState).title=(*pState).title+', '+(*pState).tch_tit
    
    ;construct detector droplist   
    if (*pState).ftype eq 0 then begin ;DCS det bank info
       tmp   = string(*(*pState).tthPtr,format='(f8.2)')
       tmp   = string(indgen((*pState).ndet)+1,format='(i3)')+tmp
       index = dm_mask_detbanks(0)
       tmp[index] = tmp[index]+'  c' & index=-1  ;central bank
       index = dm_mask_detbanks(1)
       tmp[index] = tmp[index]+'  u' & index=-1  ;upper bank
       index = dm_mask_detbanks(-1)
       tmp[index] = tmp[index]+'  l' & index=-1  ;lower bank    
    endif else begin
       tmp   = string(indgen((*pState).ndet)+1,format='(i'+dm_to_string(ceil(alog10((*pState).ndet)))+')')+'   '+dm_to_string(*(*pState).tthPtr,res=3)+string('b0'xb)
    endelse
    det_info = [det_info,tmp]
    
    if ~keyword_set(init) then begin
       widget_control,(*pState).tlb,tlb_set_title=(*(*pState).fname)[(*pState).ftype]+' Single Crystal Alignment Utility'
       widget_control,widget_info((*pState).tlb,find_by_uname='calca2but'),set_value='Calculate '+(*(*pState).a2name)[(*pState).ftype]+' Reset Value'
       widget_control,widget_info((*pState).tlb,find_by_uname='calca2rot'),set_value='Calculate '+(*(*pState).a2name)[(*pState).ftype]+' Rotation Angle'
       dm_set_droplist,widget_info((*pState).tlb,find_by_uname='detlist'),select=0,set_value=det_info
       dm_set_droplist,widget_info((*pState).tlb,find_by_uname='timelist'),select=(*pState).t_chan
       dm_set_droplist,widget_info((*pState).tlb,find_by_uname='xaxislist'),select=(*pState).xaxis,set_value=x_info
       ptr_free,(*pState).xPtr,(*pState).yPtr,(*pState).dyPtr
       (*pState).xPtr = ptr_new(/allocate_heap)  
       (*pState).yPtr = ptr_new(/allocate_heap)
       (*pState).dyPtr = ptr_new(/allocate_heap)
       (*pState).filesel->set_filter,(*(*pState).fextn)[(*pState).ftype]
    endif 
end

pro dcs_cryst_align_setfilehistory,pState,dir,files,choose=choose
    if n_elements(choose) eq 1 then begin
       (*pState).filesel->set_path,(*((*pState).file_hist[choose]))[2]
       ftype = strmid((*((*pState).file_hist[choose]))[0],6,strlen((*((*pState).file_hist[choose]))[0])-6)
       if ftype ne (*pState).ftype then begin
          wid = widget_info((*pState).tlb,find_by_uname='ftype_'+dm_to_string(ftype))
          dcs_cryst_align_changeftype,{WIDGET_BUTTON,ID:wid,TOP:(*pState).tlb,HANDLER:(*pState).tlb,SELECT:1}
       endif
       (*pState).filesel->getproperty,dFiles=dFiles
       for i=3,n_elements(*((*pState).file_hist[choose]))-1 do begin
           ind = where(dFiles eq (*((*pState).file_hist[choose]))[i],count)
           if count ne 0 then begin
              if n_elements(selected) eq 0 then selected=ind $
              else selected=[selected,ind]
           endif
       endfor
       if n_elements(selected) ne 0 then (*pState).filesel->set_file,selected,/highlight
       return
    endif
    num = n_elements((*pState).file_hist)    ;capacity
    nfiles = n_elements(files)
    if (n_elements(dir) ne 0) and (nfiles ne 0) then begin
       info = files[0] & if nfiles gt 1 then info = info+' to '+files[nfiles-1]
       for i=0,(*pState).n_history-1 do begin  ;check if this entry already exists
           if dir ne (*((*pState).file_hist[i]))[2] then continue
           if n_elements(files) ne n_elements(*((*pState).file_hist[i]))-3 then continue
           same = 1b
           for j=0, n_elements(files)-1 do $
               if files[j] ne (*((*pState).file_hist[i]))[3+j] then same=0b
           if same then return ;entry exists
       endfor
       if (*pState).n_history lt num then begin ;empty room available
          if (*pState).n_history ne 0 then $
             (*pState).file_hist[1:(*pState).n_history] = (*pState).file_hist[0:(*pState).n_history-1]
          (*pState).file_hist[0] = ptr_new(['ftype_'+dm_to_string((*pState).ftype,/int),info,dir,files])
          (*pState).n_history = (*pState).n_history+1
       endif else begin                    ;full
          ptr_free,(*pState).file_hist[num-1]
          (*pState).file_hist[1:num-1] = (*pState).file_hist[0:num-2]
          (*pState).file_hist[0] = ptr_new(['ftype_'+dm_to_string((*pState).ftype,/int),info,dir,files])
       endelse
    endif
    histmenu = widget_info((*pState).tlb,find_by_uname='fhistMenu')
    widget_control,histmenu,sensitive=((*pState).n_history gt 0)
    widget_control,widget_info((*pState).tlb,find_by_uname='clearHistory'),sensitive=((*pState).n_history gt 0)
    for i=0,num-1 do begin
        tmp = widget_info(histmenu,find_by_uname='fhist_'+dm_to_string(i,/int))
        if tmp ne 0 then widget_control,tmp,/destroy
    endfor
    for i=0,(*pState).n_history-1 do begin
        void = widget_button(histmenu,value=dm_to_string(i,/int)+' '+(*((*pState).file_hist[i]))[1],uname='fhist_'+dm_to_string(i))
    endfor
end

pro dcs_cryst_align_event,event
    compile_opt IDL2    ;,strictarrsubs
                        ;idl2=defint32,strictarr
                        ;strictarr:   [] to index array
                        ;strictarrsubs: error when out-of-range indices,IDL5.6 or after
    WIDGET_CONTROL,/HOURGLASS
    widget_control,event.handler,get_uvalue=pState

    if (*pState).enotebook then $
       IF DAVE_SET_FOCUS(EVENT) THEN RETURN

    widget_type = widget_info(event.id,/type)
    case widget_type of
       8:    tmp_value = widget_info(event.id,/droplist_select)  ;droplist
       12:   tmp_value = event.index                             ;combobox
       else: tmp_value = 0
    endcase

    ;catch and ignore all errors in this program
    catch, myerror
    if myerror ne 0 then begin
       ok = dialog_message(dialog_parent=(*pState).tlb,!error_state.msg,/error,/center)
       catch,/cancel
       return
    end
    
    dgsym  = '\deg' ;string('b0'xb)
    a2name = (*(*pState).a2name)[(*pState).ftype]
    uname  = strlowcase(widget_info(event.id,/uname))
    case uname of
       'tlb':    widget_control,event.id,scr_xsize=(*pState).geom[0],scr_ysize=(*pState).geom[1]
       'loadbut': begin
          (*pState).fit = 0
          (*pState).filesel->getproperty,path=open_path,file=open_file
          if n_elements(open_file) eq 0 then begin
             ok = dialog_message('Please select '+(*(*pState).fname)[(*pState).ftype]+' files first.',/error,dialog_parent=event.handler,/center)
             break
          endif
          e_range = (*pState).e_ran
          case (*pState).ftype of 
              0: begin  ;DCS
                 dm_load_dcsdata,open_path,open_file,ang=x,qty=y,dqty=dy,t_chan=(*pState).t_chan,e_range=e_range,$
                      parent=(*pState).tlb,duration=duration,/raw,eadjust=1,montype=0,/fissionchamber,bitflip=(*pState).bitflip
                 end
              1: begin  ;WAND
                 n_files = n_elements(open_file)
                 for i=0L,n_files-1L do begin
                     file = open_path+path_sep()+open_file[i]
                     if i eq 0 then $
                        mesg =  obj_new('dm_progress',title='Loading data',message='Loading '+file,group_leader=(*pState).tlb) $
                     else $
                        mesg->update,message='loading '+file,title='Loading...  '+strtrim(string(i+1),2)+'/'+strtrim(string(n_files),2)
                     ;read the wand file
                     dm_load_wand,file,data=data,psi=psi,temperature=temperature,error=error,group_leader=(*pState).tlb
                     if keyword_set(error) then begin
                        if obj_valid(mesg) then obj_destroy,mesg
                        if n_elements(x) eq 0 then return else break
                     endif
                     mask = where(data[*,0] lt 0,mskcount)
                     if mskcount gt 0 then data[mask,*] = 0.0
                     if i eq 0 then begin
                        x = transpose([[psi],[temperature]])
                        y = data 
                     endif else begin
                        x = [[x],[transpose([[psi],[temperature]])]]
                        y = [[y],[data]] 
                     endelse
                     if (~ obj_valid(mesg)) then break
                 endfor
                 if obj_valid(mesg) then obj_destroy,mesg
                 dy = sqrt(y)
                 end
              2: begin  ;MACS
                 n_files = n_elements(open_file)
                 for i=0L,n_files-1L do begin
                     file = open_path+path_sep()+open_file[i]
                     if i eq 0 then $
                        mesg =  obj_new('dm_progress',title='Loading data',message='Loading '+file,group_leader=(*pState).tlb) $
                     else $
                        mesg->update,message='loading '+file,title='Loading...  '+strtrim(string(i+1),2)+'/'+strtrim(string(n_files),2)
                     ;read the macs file
                     dm_load_macs,file,data,colinfo,header=header,latticeparm=latticeparm,latticeori=latticeori,error=error,group_leader=(*pState).tlb
                     if keyword_set(error) then begin
                        if obj_valid(mesg) then obj_destroy,mesg
                        if n_elements(x) eq 0 then return else break
                     endif
                     if n_elements(data) eq 0 then continue
                     ind_qty = lonarr(2,20)
                     for j=1,20 do begin
                         tmp = string(j,format='(i02)')
                         ind_qty[0,j-1] = where(colinfo eq 'spec'+tmp)
                         ind_qty[1,j-1] = where(colinfo eq 'diff'+tmp)
                     endfor
                     ind_a1 = where(colinfo eq 'a1',cnt_1)
                     ind_a2 = where(colinfo eq 'a2',cnt_2)
                     ind_a3 = where(colinfo eq 'a3',cnt_3)
                     ind_a5 = where(colinfo eq 'a5',cnt_5)
                     ind_a6 = where(colinfo eq 'a6',cnt_6)
                     ind_mr = where(colinfo eq 'monrot',cnt_m)
                     ind_ds = where(colinfo eq 'dfmdts',cnt_d)
                     ind_bs = lonarr(21)
                     for j=1,21 do begin
                         tmp = string(j,format='(i02)')
                         ind_bs[j-1] = where(colinfo eq 'monblade'+tmp,cnt_b)
                     endfor
                     ind_sx = where(colinfo eq 'smplx',cnt_x)
                     ind_sy = where(colinfo eq 'smply',cnt_y)
                     ind_sz = where(colinfo eq 'smplz',cnt_z)
                     ind_lt = where(colinfo eq 'smplltilt',cnt_l)
                     ind_ut = where(colinfo eq 'smplutilt',cnt_u)
                     ind_kn = where(colinfo eq 'kidney',cnt_n)
                     ind_wt = where(colinfo eq 'monitor')
                     ind_t  = where(colinfo eq 'temp',cnt_t)
                     if cnt_1 gt 0 then a1 = data[*,ind_a1] else a1 = fltarr(n_elements(data[*,0]))
                     if cnt_2 gt 0 then a2 = data[*,ind_a2] else a2 = fltarr(n_elements(data[*,0]))
                     if cnt_3 gt 0 then a3 = data[*,ind_a3] else a3 = fltarr(n_elements(data[*,0]))
                     if cnt_5 gt 0 then a5 = data[*,ind_a5] else a5 = fltarr(n_elements(data[*,0]))
                     if cnt_6 gt 0 then a6 = data[*,ind_a6] else a6 = fltarr(n_elements(data[*,0]))
                     if cnt_m gt 0 then mr = data[*,ind_mr] else mr = fltarr(n_elements(data[*,0]))
                     if cnt_d gt 0 then ds = data[*,ind_ds] else ds = fltarr(n_elements(data[*,0]))
                     if cnt_b gt 0 then bs = data[*,ind_bs] else bs = fltarr(n_elements(data[*,0]),21)
                     if cnt_x gt 0 then sx = data[*,ind_sx] else sx = fltarr(n_elements(data[*,0]))
                     if cnt_y gt 0 then sy = data[*,ind_sy] else sy = fltarr(n_elements(data[*,0]))
                     if cnt_z gt 0 then sz = data[*,ind_sz] else sz = fltarr(n_elements(data[*,0]))
                     if cnt_l gt 0 then sl = data[*,ind_lt] else sl = fltarr(n_elements(data[*,0]))
                     if cnt_u gt 0 then su = data[*,ind_ut] else su = fltarr(n_elements(data[*,0]))
                     if cnt_n gt 0 then kn = data[*,ind_kn] else kn = fltarr(n_elements(data[*,0]))
                     if cnt_t gt 0 then temperature = data[*,ind_t] else temperature = fltarr(n_elements(data[*,0]))
                     qty = transpose([[[data[*,ind_qty[0,*]]]],[[data[*,ind_qty[1,*]]]]])
                     err = sqrt(qty)
                     if i eq 0 then moncnt = data[0,ind_wt[0]]
                     monfac = moncnt/(1>data[*,ind_wt])
                     for j=0L,n_elements(monfac)-1 do begin
                         qty[*,*,j] = qty[*,*,j]*monfac[j]
                         err[*,*,j] = err[*,*,j]*monfac[j]
                     endfor
                     if n_elements(x) eq 0 then begin
                        duration = moncnt 
                        ;figure out the monitor is time or monitor count
                        yn = stregex(header, '#reference',/boolean,/fold_case)
                        index = where(yn,count)
                        if count gt 0 then begin
                           tmp = strlowcase(strsplit(header[index[0]],' '+string(9b),/extract))
                           index = where(colinfo eq tmp[n_elements(tmp)-1],count)
                           if (count gt 0) and (tmp[n_elements(tmp)-1] eq 'time') then begin
                              ind_tm = where(colinfo eq 'time')
                              duration = -abs(data[0,ind_tm[0]]) ;negative value to denote time
                           endif
                        endif
                        x  = transpose([[a3],[kn],[a1],[a2],[a5],[a6],[mr],[ds],[bs],[sx],[sy],[sz],[sl],[su],[temperature],[data[*,ind_wt]]])
                        y  = qty
                        dy = err
                        (*(*pState).infoPtr)[1:12] = [latticeparm,latticeori]
                     endif else begin
                        x  = [[x],[transpose([[a3],[kn],[a1],[a2],[a5],[a6],[mr],[ds],[bs],[sx],[sy],[sz],[sl],[su],[temperature],[data[*,ind_wt]]])]]
                        y  = [[[y]],[[qty]]] 
                        dy = [[[dy]],[[err]]]
                     endelse
                     if (~ obj_valid(mesg)) then break
                 endfor
                 if obj_valid(mesg) then obj_destroy,mesg
                 end   
              else:
          endcase
          if n_elements(e_range) ne 0 then (*pState).e_ran = e_range
          *(*pState).xPtr  = temporary(x)
          *(*pState).yPtr  = temporary(y)
          *(*pState).dyPtr = temporary(dy)
          if n_elements(duration) ne 0 then (*pState).duration = duration
          dcs_cryst_align_plot,pState
          dcs_cryst_align_setfilehistory,pState,open_path,open_file
         end
       'optnbitflip': begin
          (*pState).bitflip = 1-(*pState).bitflip
          dm_set_button,event.id,(*pState).bitflip,onstring='Bitflip Error Correction Enabled',offstring='Bitflip Error Correction Disabled'
         end  
       'optnindvdet': begin
          (*pState).indvdet = 1-(*pState).indvdet
          dm_set_button,event.id,(*pState).indvdet,onstring='Show Individual Detector',offstring='Sum Detectors'
          widget_control,widget_info((*pState).tlb,find_by_uname='fitbut'),sensitive=(~(*pState).indvdet or (n_elements(*(*pState).detPtr) eq 1))
          dcs_cryst_align_plot,pState
         end
       'optnshowfit': begin
          (*pState).showfit = 1-(*pState).showfit
          dm_set_button,event.id,(*pState).showfit,onstring='Show Fitting Parameters',offstring='Hide Fitting Parameters'
          dcs_cryst_align_plot,pState
         end  
       'macs_det_spec': if (*pState).y_typ ne 0 then begin
          (*pState).y_typ = 0
          diff = widget_info((*pState).tlb,find_by_uname='macs_det_diff')
          dm_toggle_menubut,check=event.id,uncheck=diff
          tmp = (*pState).title
          ind = strpos(strlowcase(tmp),'diff ')
          if ind ne -1 then begin
             strput,tmp,'spec',ind
             (*pState).title = tmp
          endif
          dcs_cryst_align_plot,pState
         end
       'macs_det_diff': if (*pState).y_typ ne 1 then begin
          (*pState).y_typ = 1
          spec = widget_info((*pState).tlb,find_by_uname='macs_det_spec')
          dm_toggle_menubut,check=event.id,uncheck=spec
          tmp = (*pState).title
          ind = strpos(strlowcase(tmp),'spec ')
          if ind ne -1 then begin
             strput,tmp,'diff',ind
             (*pState).title = tmp
          endif
          dcs_cryst_align_plot,pState
         end
       'fitbut':  begin
          if n_elements(*(*pState).yPtr) le 4 then break
          (*pState).fit = 1
          dcs_cryst_align_plot,pState
         end
       'savebut': begin
          if n_elements(*(*pState).xPtr) eq 0 then break
          if dm_to_number(!version.release) ge 8.0 then $          
             ftypename = ['Postscript','PDF','JPEG','GIF','TIFF','BMP','PNG','PPM'] $
          else $
             ftypename = ['Postscript','JPEG','GIF','TIFF','BMP','PNG','PPM']
          ftype = dm_dialog_input('file type:',default=0,droplist_content=ptr_new(ftypename),is_droplist=[1],$
              /return_number,title='Please choose',dialog_parent=(*pState).tlb,xsize=80,cancel=cancel)
          if keyword_set(cancel) then return
          ftypename[0] = 'PS'    
          (*pState).plotWin->saveas,ftypename[ftype]
         end
       'databut':  begin
          if n_elements(*(*pState).xPtr) eq 0 then break
          path = (*pState).workDir
          file = dm_choose_file('txt',/write,dialog_parent=(*pState).tlb,title='Save plot data as an ASCII file.',path=path)
          if strlen(file) eq 0 then break
          if file_test(path,/directory) then (*pState).workDir=path
          openw,unit,file,/get_lun,error=openerr
          if openerr ne 0 then begin
             ok = dialog_message("Can't write in "+file[0],/error,dialog_parent=(*pState).tlb,/center)
          endif else begin
             dcs_cryst_align_plot,pState,/dataonly,xs=x,ys=y,dys=dy  ;get data
             widget_control,/hourglass  ;busy signal
             for i=0L,n_elements(x)-1L do $
                 printf,unit,x[i],' ',y[i],' ',dy[i]
             free_lun,unit
          endelse
         end
       'printbut': begin
          if n_elements(*(*pState).xPtr) eq 0 then break
          (*pState).plotWin->saveas,'printer'
         end
       'exitbut': begin
          widget_control,event.handler,/destroy
          return
         end
       'detlist': begin
          old = *(*pState).detPtr
          (*pState).tth = [!values.f_nan,!values.f_nan]
          case tmp_value of
              0: begin
                 *(*pState).detPtr = indgen((*pState).ndet)
                 (*pState).det_tit = (['','',(['spec ','diff '])[(*pState).y_typ]])[(*pState).ftype]+'det=all'
                 end
              1: begin
                 n_index = n_elements(*(*pState).detPtr)
                 maxdet  = max(*(*pState).detPtr,min=mindet)
                 if (n_index eq (maxdet-mindet+1)) and (n_index gt 3) then $
                    default = dm_to_string(mindet+1)+'-'+dm_to_string(maxdet+1) $
                 else $
                    default = dm_to_string(*(*pState).detPtr+1,separator=',')
                 tmp = dm_dialog_input(dialog_parent=event.handler,'Det num:',default=default,title='Specifying detectors',$
                       info='Detector number starts from 1 to '+dm_to_string((*pState).ndet)+'. eg:'+(['371,374 or 370-374 c','120,121,122 or 120-122','6,7,8 or 6-8'])[(*pState).ftype],xsize=200)
                 (*pState).det_tit = (['','',(['spec ','diff '])[(*pState).y_typ]])[(*pState).ftype]+'det='+tmp
                 if strlen(tmp) ne 0 then begin
                    ;search for :- range detectors
                    len = [1]
                    det_tmp = -1L
                    while (len[0] gt 0) do begin
                        tmp1 = stregex(tmp,'([0-9]+)[ '+string(9b)+']*[:-][ '+string(9b)+']*([0-9]+)[ '+string(9b)+']*([cCuUlL]*)',/subexpr,length=len)
                        if len[0] gt 0 then begin
                           tmp2 = dm_to_number(strmid(tmp,tmp1[1],len[1]),/long)
                           tmp3 = dm_to_number(strmid(tmp,tmp1[2],len[2]),/long)
                           if len[3] gt 0 then status = strlowcase(strmid(tmp,tmp1[3],1)) else status=''
                           tmp4 = min([tmp2,tmp3])+lindgen(abs(tmp3-tmp2)+1)
                           if status ne '' then $
                              tmp4 = dm_common(tmp4,dm_mask_detbanks((where(['l','c','u'] eq status))[0]-1)+1,/no_copy)
                           det_tmp = [det_tmp,tmp4]
                           tmp = strmid(tmp,0,tmp1[0])+strmid(tmp,tmp1[0]+len[0],strlen(tmp)-tmp1[0]-len[0])
                        endif
                    endwhile 
                    ;individual detectors
                    tmp1 = strsplit(tmp,'[a-zA-Z '+string(9b)+',:-]',/regex,/extract)
                    tmp2 = dm_to_number(tmp1)
                    ind2 = where(finite(tmp2),count2)
                    if count2 gt 0 then det_tmp = [det_tmp,dm_to_number(tmp1[ind2],/long)]
                    index = where(det_tmp gt 0,count)
                    if count gt 0 then dets = det_tmp[index]-1
                 endif  
                 ndet = n_elements(dets)
                 if ndet gt 0 then dets = dets[uniq(dets,sort(dets))]
                 if n_elements(dets) ne ndet then (*pState).det_tit = (['','',(['spec ','diff '])[(*pState).y_typ]])[(*pState).ftype]+'det='+dm_to_string(dets+1,separator=',')
                 ndet = n_elements(dets)
                 if (ndet eq 0) or (ndet eq (*pState).ndet) then begin
                    *(*pState).detPtr = indgen((*pState).ndet)
                    (*pState).det_tit = (['','',(['spec ','diff '])[(*pState).y_typ]])[(*pState).ftype]+'det=all'
                    dm_set_droplist,event.id,select=0
                 endif else begin
                    *(*pState).detPtr = dets
                    maxdet = max(dets,min=mindet)
                    if (ndet eq (maxdet-mindet+1)) and (ndet gt 3) then $
                       (*pState).det_tit = (['','',(['spec ','diff '])[(*pState).y_typ]])[(*pState).ftype]+'det='+dm_to_string(mindet+1)+'-'+dm_to_string(maxdet+1)
                    (*pState).tth[0] = mean((*(*pState).tthPtr)[dets])
                    tth = dm_to_string((*pState).tth[0],resolution=3)+dgsym
                    if ndet gt 1 then tth = '(<2th>='+tth+')' else tth='(2th='+tth+')'
                    (*pState).det_tit = (*pState).det_tit+tth
                 endelse
                 end
              2: begin
                 if (*pState).ftype eq 0 then begin
                    *(*pState).detPtr = dm_mask_detbanks(0)
                    (*pState).det_tit = 'det=central'
                 endif else if (*pState).ftype eq 2 then begin
                    *(*pState).detPtr = tmp_value-3  ;monitor
                    (*pState).det_tit = ''
                 endif else begin
                    *(*pState).detPtr = tmp_value-2
                    (*pState).tth[0]  = (*(*pState).tthPtr)[tmp_value-2]
                    tth = dm_to_string((*pState).tth[0])+dgsym
                    (*pState).det_tit = (['','',(['spec ','diff '])[(*pState).y_typ]])[(*pState).ftype]+'det='+strtrim(string(tmp_value-1),2)+'(2th='+tth+')'
                 endelse
                 end
              3: begin
                 if (*pState).ftype eq 0 then begin
                    *(*pState).detPtr = dm_mask_detbanks(1)
                    (*pState).det_tit = 'det=upper'
                 endif else if (*pState).ftype eq 2 then begin
                    *(*pState).detPtr = tmp_value-3
                    (*pState).det_tit = ''
                 endif else begin
                    *(*pState).detPtr = tmp_value-2
                    (*pState).tth[0]  = (*(*pState).tthPtr)[tmp_value-2]
                    tth = dm_to_string((*pState).tth[0])+dgsym
                    (*pState).det_tit = (['','',(['spec ','diff '])[(*pState).y_typ]])[(*pState).ftype]+'det='+strtrim(string(tmp_value-1),2)+'(2th='+tth+')'
                 endelse   
                 end
              4: begin
                 if (*pState).ftype eq 0 then begin
                    *(*pState).detPtr = dm_mask_detbanks(-1)
                    (*pState).det_tit = 'det=lower'
                 endif else if (*pState).ftype eq 2 then begin
                    *(*pState).detPtr = tmp_value-3
                    (*pState).det_tit = ''
                 endif else begin
                    *(*pState).detPtr = tmp_value-2
                    (*pState).tth[0]  = (*(*pState).tthPtr)[tmp_value-2]
                    tth = dm_to_string((*pState).tth[0])+dgsym
                    (*pState).det_tit = (['','',(['spec ','diff '])[(*pState).y_typ]])[(*pState).ftype]+'det='+strtrim(string(tmp_value-1),2)+'(2th='+tth+')'
                 endelse
                 end
              5: begin
                 if (*pState).ftype eq 0 then begin
                    *(*pState).detPtr = 913
                    (*pState).det_tit = 'FC2'
                 endif else if (*pState).ftype eq 2 then begin
                    *(*pState).detPtr = tmp_value-3
                    (*pState).det_tit = ''
                 endif else begin
                    *(*pState).detPtr = tmp_value-2
                    (*pState).tth[0]  = (*(*pState).tthPtr)[tmp_value-2]
                    tth = dm_to_string((*pState).tth[0])+dgsym
                    (*pState).det_tit = (['','',(['spec ','diff '])[(*pState).y_typ]])[(*pState).ftype]+'det='+strtrim(string(tmp_value-1),2)+'(2th='+tth+')'
                 endelse
                 end
              6: begin
                 if (*pState).ftype eq 0 then begin
                    *(*pState).detPtr = 914
                    (*pState).det_tit = 'FC3'
                 endif else if (*pState).ftype eq 2 then begin
                    *(*pState).detPtr = tmp_value-3
                    (*pState).det_tit = ''
                 endif else begin
                    *(*pState).detPtr = tmp_value-2
                    (*pState).tth[0]  = (*(*pState).tthPtr)[tmp_value-2]
                    tth = dm_to_string((*pState).tth[0])+dgsym
                    (*pState).det_tit = (['','',(['spec ','diff '])[(*pState).y_typ]])[(*pState).ftype]+'det='+strtrim(string(tmp_value-1),2)+'(2th='+tth+')'
                 endelse
                 end
              else:begin
                 if (*pState).ftype eq 0 then begin
                    *(*pState).detPtr = tmp_value-7
                    (*pState).tth[0]  = (*(*pState).tthPtr)[tmp_value-7]
                    tth = dm_to_string((*pState).tth[0])+dgsym
                    (*pState).det_tit = 'det='+strtrim(string(tmp_value-6),2)+'(2th='+tth+')'
                 endif else if (*pState).ftype eq 2 then begin
                    *(*pState).detPtr = tmp_value-3
                    (*pState).det_tit = ''
                 endif else begin
                    *(*pState).detPtr = tmp_value-2
                    (*pState).tth[0]  = (*(*pState).tthPtr)[tmp_value-2]
                    tth = dm_to_string((*pState).tth[0])+dgsym
                    (*pState).det_tit = (['','',(['spec ','diff '])[(*pState).y_typ]])[(*pState).ftype]+'det='+strtrim(string(tmp_value-1),2)+'(2th='+tth+')'
                 endelse
                 end
          endcase
          if (*pState).ftype eq 0 then $
             (*pState).title = (*pState).det_tit+', '+(*pState).tch_tit $
          else $
             (*pState).title = (*pState).det_tit
          n_old = n_elements(old)
          n_new = n_elements(*(*pState).detPtr)
          comm  = dm_common(old,*(*pState).detPtr)
          same  = (n_old eq n_new) and (n_old eq n_elements(comm)) and (comm[0] ne -1)
          old   = -1 & comm = -1
          widget_control,widget_info((*pState).tlb,find_by_uname='fitbut'),sensitive=(~(*pState).indvdet or (n_elements(*(*pState).detPtr) eq 1))
          if (n_elements(*(*pState).yPtr) ne 0) and (~ same) then begin   ;reload data
             dcs_cryst_align_plot,pState
          endif
         end
       'timelist':   if ((*pState).t_chan ne tmp_value) or (tmp_value eq 2) then begin
          (*pState).t_chan = tmp_value
          if (*pState).t_chan eq 0 then (*pState).tch_tit='tchan=all' $
          else if (*pState).t_chan eq 1 then (*pState).tch_tit='tchan=elastic' $
          else begin
             default = dm_to_string((*pState).e_ran)
             new     = dm_dialog_input(['from:','to:'],title='Energy range',xsize=100,default=default,/float,$
                       dialog_parent=(*pState).tlb,info='In units of meV')
             (*pState).t_chan = 1
             (*pState).e_ran[*] = !values.f_nan
             if total(finite(new)) eq 2 then $
                if new[1] gt new[0] then begin
                   (*pState).e_ran   = new
                   (*pState).t_chan  = 2
                   (*pState).tch_tit = 'E=['+dm_to_string(new[0])+','+dm_to_string(new[1])+']meV'
                endif
             if (*pState).t_chan eq 1 then begin
                ok = dialog_message('Invalid energy range. Elastic peak option is used.',dialog_parent=(*pState).tlb,/center)
                dm_set_droplist,event.id,select=1
                (*pState).tch_tit = 'tchan=elastic'
             endif
          endelse
          (*pState).title = (*pState).det_tit+', '+(*pState).tch_tit
          (*pState).filesel->getproperty,file=open_file
          if n_elements(open_file) ne 0 then begin ;reload data
             tlb  = (*pState).tlb
             load = widget_info((*pState).tlb,find_by_uname='loadbut')
             dcs_cryst_align_event,{WIDGET_BUTTON,ID:load,TOP:tlb,HANDLER:tlb,SELECT:1}
             return
          endif
         endif
       'xaxislist':if (*pState).xaxis ne tmp_value then begin
          (*pState).xaxis = tmp_value
          (*pState).fit   = 0
          if n_elements(*(*pState).xPtr) ne 0 then dcs_cryst_align_plot,pState
         end
       'calctthbut':begin
          default = (float(*(*pState).infoPtr))[0:9]
          if (*pState).ftype eq 2 then begin  ;macs, use energy 
             if finite(default[0]) then default[0] = (81.8042d)/default[0]/default[0]
             info = dm_dialog_input(['E (meV):','a ('+string('c5'XB)+'):','b ('+string('c5'XB)+'):',$
                    'c ('+string('c5'XB)+'):','alpha:','beta:','gamma:','H:','K:','L:'],default=default,/float,$
                    dialog_parent=(*pState).tlb,xsize=80,title='Calculate 2theta',/e2lambda,cancel=cancel)
          endif else begin                    ;use wavelength
             info = dm_dialog_input(['lambda ('+string('c5'XB)+'):','a ('+string('c5'XB)+'):','b ('+string('c5'XB)+'):',$
                    'c ('+string('c5'XB)+'):','alpha:','beta:','gamma:','H:','K:','L:'],default=default,/float,$
                    dialog_parent=(*pState).tlb,xsize=80,title='Calculate 2theta',/lambda2e,cancel=cancel)
          endelse
          if keyword_set(cancel) then return
          if total(finite(info)) eq n_elements(default) then begin
             if (*pState).ftype eq 2 then begin
                tmp = dm_check_macskidney(1,5,limits=limits)
                llim = interpol(limits[*,1],limits[*,0],info[0])  ;kidney lower limit
                ulim = interpol(limits[*,2],limits[*,0],info[0])  ;kidney upper limit
                info[0] = sqrt((81.8042d)/info[0])
             endif
             (*(*pState).infoPtr)[0:9] = double(info)
             ki = 2.*!dpi/info[0]
             q  = dcs_cryst_align_qlength(info[7:9],info[1:6])
             if q/2. lt ki then begin
                mesg = ''
                tth  = asin(q/2./ki)*2/!dtor
                if (*pState).ftype eq 2 then begin
                   det_l = (([tth,-tth]-llim+76.)/8)<19
                   det_u = (([tth,-tth]-ulim+76.)/8)>0
                   if det_l[0] ge det_u[0] then begin
                      ptai = 1>(round((det_l[0]+det_u[0])/2)+1)<20
                   endif else if det_l[1] ge det_u[1] then begin
                      tth = -tth
                      ptai = 1>(round((det_l[1]+det_u[1])/2)+1)<20
                   endif else $
                      mesg = 'It is outside the detector range.'
                   if n_elements(ptai) ne 0 then begin   
                      kidney = tth+76-8*(ptai-1)
                      mesg = 'PTAI='+dm_to_string(ptai)+', kidney='+dm_to_string(kidney,resolution=3)+string('b0'xb)+'.'
                   endif
                endif else begin
                   idet = where(*(*pState).tthPtr ge tth,count)
                   if tth lt min(*(*pState).tthPtr) then count = 0
                   if count ne 0 then begin
                      if idet[0]-1 gt 0 then begin
                         if abs((*(*pState).tthPtr)[idet[0]-1]-tth) lt abs((*(*pState).tthPtr)[idet[0]]-tth) then $
                            idet[0] = idet[0]-1
                      endif
                      tmp = [idet[0],((n_elements(*(*pState).tthPtr)-1)<(idet[0]+1)),(0>(idet[0]-1))] & count = 0
                      if (*pState).ftype eq 0 then idet = dm_common(tmp,dm_mask_detbanks(0),count)
                      if count eq 0 then begin
                         mesg = 'The corresponding detector number is '+dm_to_string(tmp[0]+1,/int)+'.'  
                      endif else begin
                         mesg = 'The corresponding central bank detector number is '+dm_to_string(idet[0]+1,/int)+'.'
                      endelse
                      if (*pState).ftype eq 0 then mesg = [mesg,'(Detector number starts from 1. DCS liveplot detector number starts from 0.)']   
                   endif else $
                      mesg = 'It is outside the detector range.'
                endelse
                tth  = dm_to_string(tth,resolution=3)+string('b0'xb)
                mesg = ['Two theta angle for ['+dm_to_string(info[7:9],separator=',')+'] is '+tth+'.',mesg]
             endif else begin
                lmax = 4.*!dpi/q
                if (*pState).ftype eq 2 then begin
                   emin = 81.8042/lmax/lmax
                   mesg = 'Incident energy must be great than '+dm_to_string(emin,resolution=2)+' meV to view ['+dm_to_string(info[7:9],separator=',')+'] reflection.'
                endif else begin
                   lmax = dm_to_string(lmax,resolution=3)+' '+string('c5'XB)
                   mesg = 'Incident wavelength must be smaller than '+lmax+' to view ['+dm_to_string(info[7:9],separator=',')+'] reflection.'
                endelse
             endelse
             ok = dialog_message(mesg,/info,dialog_parent=(*pState).tlb,title='Result',/center)
          endif
         end
       'calclattbut':begin
          default = [abs((*pState).tth[(*pState).ftype eq 2]),float((*(*pState).infoPtr)[[0,4,5,6,7,8,9]])]
          if (*pState).ftype eq 2 then begin  ;macs, use energy 
             if finite(default[1]) then default[1] = (81.8042d)/default[1]/default[1]
             info = dm_dialog_input(['2theta:','E (meV):','alpha:','beta:','gamma:','H:','K:','L:'],title='calculate',$
                    info='assuming a=b=c',default=default,/float,dialog_parent=(*pState).tlb,xsize=80,cancel=cancel)
          endif else begin
             info = dm_dialog_input(['2theta:','lambda ('+string('c5'XB)+'):','alpha:','beta:','gamma:','H:','K:','L:'],title='calculate',$
                    info='assuming a=b=c',default=default,/float,dialog_parent=(*pState).tlb,xsize=80,cancel=cancel)
          endelse
          if keyword_set(cancel) then return
          if total(finite(info)) eq 8 then begin
             info = double(info)
             if (*pState).ftype eq 2 then info[1] = sqrt((81.8042d)/info[1])
             qlen = dcs_cryst_align_qlength(info[5:7],[1.0,1.0,1.0,info[2:4]])
             a    = qlen*info[1]/4./!dpi/sin(abs(info[0])/2*!dtor)
             ok   = dialog_message("The lattice paramter is "+dm_to_string(a,resolution=4)+' '+string('c5'xB)+' assuming a=b=c.',$
                    /info,dialog_parent=(*pState).tlb,title='Result',/center)
          endif
         end
       'calca2but':begin
          case (*pState).ftype of 
               0:    info = a2name+['=90-theta, 2theta>0','=-(90+theta), 2theta<0']
               1:    info = a2name+'=theta-90'
               2:    info = a2name+['=theta, 2theta>0','=180+theta, 2theta<0']
               else: info = ''
          endcase
          tth = dm_dialog_input('two theta:',default=(*pState).tth[(*pState).ftype eq 2],info=info,xsize=80,$
                dialog_parent=(*pState).tlb,title='Calculate '+a2name+' reset value',cancel=cancel)
          if keyword_set(cancel) then return
          if finite(tth) then begin
             if (*pState).ftype eq 1 then tth = -tth  ;WAND
             if tth ge 0 then a2 = 90.0-tth/2. else a2=-(90.0+tth/2.)
             if (*pState).ftype eq 2 then a2 = 90-a2  ;MACS
             mesg = 'Set '+a2name+' to '+dm_to_string(a2,resolution=3)+string('b0'xb)+'.'
             ok = dialog_message(mesg,/info,dialog_parent=(*pState).tlb,title='Result',/center)
          endif
         end
       'calca2rot':begin
          default = (float(*(*pState).infoPtr))[0:12]
          if (*pState).ftype eq 2 then begin  ;macs, use energy 
             if finite(default[0]) then default[0] = (81.8042d)/default[0]/default[0]
             info = dm_dialog_input(['E (meV):','a ('+string('c5'XB)+'):','b ('+string('c5'XB)+'):',$
                    'c ('+string('c5'XB)+'):','alpha:','beta:','gamma:','H1:','K1:','L1:','H2:','K2:','L2:'],$
                    default=default,/float,dialog_parent=(*pState).tlb,xsize=80,title='Calculate '+a2name+' Rotation',$
                    info='1->2 counterclockwise',cancel=cancel)
          endif else begin
             info = dm_dialog_input(['lambda ('+string('c5'XB)+'):','a ('+string('c5'XB)+'):','b ('+string('c5'XB)+'):',$
                    'c ('+string('c5'XB)+'):','alpha:','beta:','gamma:','H1:','K1:','L1:','H2:','K2:','L2:'],$
                    default=default,/float,dialog_parent=(*pState).tlb,xsize=80,title='Calculate '+a2name+' Rotation',$
                    info='1->2 counterclockwise',cancel=cancel)
          endelse
          if keyword_set(cancel) then return
          if total(finite(info)) eq n_elements(default) then begin
             if (*pState).ftype eq 2 then begin
                tmp = dm_check_macskidney(1,5,limits=limits)
                llim = interpol(limits[*,1],limits[*,0],info[0])  ;kidney lower limit
                ulim = interpol(limits[*,2],limits[*,0],info[0])  ;kidney upper limit
                info[0] = sqrt((81.8042d)/info[0])
             endif
             (*(*pState).infoPtr)[0:12] = double(info)
             ki = 2.*!dpi/info[0]
             q1 = dcs_cryst_align_qlength(info[7:9],info[1:6])
             q2 = dcs_cryst_align_qlength(info[10:12],info[1:6])
             if max([q1,q2])/2. lt ki then begin
                mesg = ''
                tth1 = asin(q1/2./ki)*2/!dtor
                tth2 = asin(q2/2./ki)*2/!dtor
                if (*pState).ftype eq 2 then begin
                   det_l = (([tth1,-tth1]-llim+76.)/8)<19
                   det_u = (([tth1,-tth1]-ulim+76.)/8)>0
                   if det_l[0] lt det_u[0] then begin
                      if det_l[1] ge det_u[1] then tth1 = -tth1 $ 
                      else mesg = 'Two theta angle for ['+dm_to_string(info[7:9],separator=',')+'] is outside the detector range.'
                   endif
                   if strlen(mesg) eq 0 then begin
                      det_l = (([tth2,-tth2]-llim+76.)/8)<19
                      det_u = (([tth2,-tth2]-ulim+76.)/8)>0
                      if det_l[0] lt det_u[0] then begin
                         if det_l[1] ge det_u[1] then tth2 = -tth2 $ 
                         else mesg = 'Two theta angle for ['+dm_to_string(info[10:12],separator=',')+'] is outside the detector range.'
                      endif  
                   endif 
                endif else begin
                   idet = where(*(*pState).tthPtr ge max([tth1,tth2]),count)
                   if count ne 0 then begin
                      if (*pState).ftype eq 1 then begin
                         tth1 = -tth1 & tth2 = -tth2
                      endif
                   endif else $
                      mesg = 'The vectors are beyond the detector range.'
                endelse
                if strlen(mesg) eq 0 then begin
                   abet = dcs_cryst_align_angle(info[7:9],info[10:12],info[1:6]) ;angle between q1 and q2
                   if tth1 ge 0 then ang1 = 90.0-tth1/2. else ang1 = -(90.0+tth1/2.)
                   if tth2 ge 0 then ang2 = 90.0-tth2/2. else ang2 = -(90.0+tth2/2.)
                   ang2 = ang2-abet
                   if (*pState).ftype eq 2 then begin
                      ang1 = 90-ang1
                      ang2 = 90-ang2
                   endif
                   mesg = a2name+' needs to rotate '+dm_to_string(ang2-ang1,resolution=2)+string('b0'xb)+'.'
                endif
             endif else begin
                lmax = 4.*!dpi/(max([q1,q2]))
                if (*pState).ftype eq 2 then begin
                   mesg = 'Incident energy must be greater than '+dm_to_string(81.8042/lmax/lmax,resolution=3)+' meV.'
                endif else begin
                   mesg = 'Incident wavelength must be smaller than '+dm_to_string(lmax,resolution=3)+' '+string('c5'XB)+'.'
                endelse
             endelse
             ok = dialog_message(mesg,/info,dialog_parent=(*pState).tlb,title='Result',/center)
          endif
         end
       'e2lambdabut':   dcs_cryst_align_e2lambda,event,lambda=float((*(*pState).infoPtr)[0]),dcs=((*pState).ftype eq 0)
       'kidlimitbut':   dcs_cryst_align_kidlimit,event,bgcolor=(*pState).bgcolor
       'clearhistory':begin
          ptr_free,(*pState).file_hist
          (*pState).n_history = 0
          dcs_cryst_align_setfilehistory,pState
         end
       else: begin
          choose = dm_to_number(strmid(uname,6,strlen(uname)-6),/int) 
          uname  = strmid(uname,0,5) 
          if uname eq 'ftype' then dcs_cryst_align_changeftype,event                     ;change file type
          if uname eq 'fhist' then dcs_cryst_align_setfilehistory,pState,choose=choose   ;file history
         end
    endcase
end

pro dcs_cryst_align_Exit,tlb
    widget_control,tlb,get_uvalue=pState,/no_copy
    obj_destroy,(*pState).filesel
    obj_destroy,(*pState).plotWin
    cd,(*pState).root
    ptr_free,(*pState).fname,(*pState).fextn,(*pState).a2name,(*pState).xtit,(*pState).file_hist
    ptr_free,(*pState).detPtr,(*pState).xPtr,(*pState).yPtr,(*pState).dyPtr,(*pState).tthPtr,(*pState).infoPtr
    ptr_free,pState
    widget_control,tlb,/destroy
end

;initparms=[a,b,c,alpha,beta,gamma,uh,uk,ul,vh,vk,vl]
pro dcs_cryst_align,event,initparms,workdir=workdir,datadir=datadir,wand=wand,bgcolor=bgcolor,macs=macs
    state={group_leader:                 0L, $  ;group leader
           tlb:                          0L, $  ;top level base
           ftype:                        0s, $  ;instrument type 0-dcs 1-wand 2-macs
           fname:   ptr_new(/allocate_heap), $  ;instrument name 
           fextn:   ptr_new(/allocate_heap), $  ;file extension
           a2name:  ptr_new(/allocate_heap), $  ;'A2' for DCS, 'Psi' for WAND
           geom:                    [0L,0L], $  ;geometry
           filesel:               obj_new(), $  ;file selector
           file_hist:            ptrarr(10), $  ;save the latest 10 file selections
           n_history:                    0s, $  ;number of histories
           root:                         '', $  ;starting directory
           infoPtr: ptr_new(/allocate_heap), $  ;store the info for calculating 2theta
                                             $  ;[lambda,a,b,c,alpha,beta,gamma,h1,k1,l1,h2,k2,l2]
           fit:                          0b, $  ;flag for fit
           showfit:                      0b, $  ;flag for showing fitting parameters
           ndet:                         0L, $  ;number of detectors
           bitflip:                      1b, $  ;flag for checking the bitflip error
           indvdet:                      0b, $  ;flag for showing individual detectors
           tthPtr:  ptr_new(/allocate_heap), $  ;save the two theta value
           detPtr:  ptr_new(/allocate_heap), $  ;pointer to detector array
           tth:                     [0e,0e], $  ;save the tth vaule [det list 2th, macs actual 2th]
           t_chan:                       1s, $  ;0-all 1-elastic 2-specify
           e_ran:                   [0e,0e], $  ;energy range for t_chan=2
           xaxis:                        0s, $  ;xaxis number
           y_typ:                        0s, $  ;for macs, 0-spec 1-diff
           xtit:    ptr_new(/allocate_heap), $  ;xtit strings
           xPtr:    ptr_new(/allocate_heap), $  ;pointer to x data
           yPtr:    ptr_new(/allocate_heap), $  ;pointer to y data
           dyPtr:   ptr_new(/allocate_heap), $  ;pointer to dy data
           bgcolor:                      '', $  ;save plot window background color
           plotWin:               obj_new(), $  ;draw widget
           duration:                     0L, $  ;duration of measurement in secs
           det_tit:                      '', $  ;output detector string
           tch_tit:                      '', $  ;output time channel string
           title:                        '', $  ;plot title
           workDir:                      '', $  ;work directory
           datDir:                       '', $  ;data directory
           enotebook:                    0b  $  ;use Rob's enotebook
    }
    registerName = 'dcs_cryst_align'
    cd,current = current
    state.root = current
    state.workDir = current
    state.datDir  = current
    if n_elements(workdir) ne 0 then state.workDir=workdir[0]
    if n_elements(datadir) ne 0 then state.datDir=datadir[0]
    if n_elements(bgcolor) ne 0 then state.bgcolor=bgcolor[0] else state.bgcolor = 'white'

    *state.infoPtr = dblarr(13) & (*state.infoPtr)[*] = !values.d_nan
    if (n_elements(initparms) le 12) and (n_elements(initparms) ne 0) then $
       (*state.infoPtr)[1:n_elements(initparms)] = initParms
    
    ;handle instrument choice. To add a new instrument, modify dcs_cryst_align_changeftype
    if keyword_set(wand) then state.ftype = 1 
    if keyword_set(macs) then state.ftype = 2
    
    pState = ptr_new(state,/no_copy)
    
    dcs_cryst_align_changeftype,/init,pState=pState,det_info=det_info,x_info=x_info

    if n_elements(event) ne 0 then begin
       (*pState).group_leader = event.top
       (*pState).tlb  = widget_base(title=(*(*pState).fname)[(*pState).ftype]+' Single Crystal Alignment Utility',/column,kill_notify='dcs_cryst_align_Exit',/tlb_size_event,$
         group_leader=event.top,map=0,mbar=bar,uname='tlb')
    endif else $
       (*pState).tlb  = widget_base(title=(*(*pState).fname)[(*pState).ftype]+' Single Crystal Alignment Utility',/column,kill_notify='dcs_cryst_align_Exit',/tlb_size_event,$
         map=0,mbar=bar,uname='tlb')
    
    ;file menu
    filemenu  = widget_button(bar,value='File',/menu)
    ftypemenu = widget_button(filemenu,value='File Type',/menu,uname='ftypeMenu')
    fhistmenu = widget_button(filemenu,value='Recent Files',/menu,uname='fhistMenu',sensitive=0,/separator)
    clearhist = widget_button(filemenu,value='Clear File History',uname='clearHistory',sensitive=0)
    fileExit  = widget_button(filemenu,value='Exit',uname='exitbut',/separator)
    for i=0,n_elements(*(*pState).fname)-1 do $
        void = dm_widget_button(ftypemenu,value=(*(*pState).fname)[i],uname='ftype_'+dm_to_string(i),set_button=((*pState).ftype eq i))
    
    ;option menu
    optnmenu    = widget_button(bar,value='Option',/menu,uname='optnMenu')   
    optnBitflip = dm_widget_button(optnmenu,value='Check Bitflip Error',$
               set_button=(*pState).bitflip,onstring='Bitflip Error Correction Enabled',$
               offstring='Bitflip Error Correction Disabled',uname='optnbitflip',sensitive=((*pState).ftype eq 0))
    optnIndvDet = dm_widget_button(optnmenu,value='Show Individual Detector',$
               set_button=(*pState).indvdet,onstring='Show Individual Detector',$
               offstring='Sum Detectors',uname='optnindvdet')   
    optnShowFit = dm_widget_button(optnmenu,value='Show Fitting Parameters',$
               set_button=(*pState).showfit,onstring='Show Fitting Parameters',$
               offstring='Hide Fitting Parameters',uname='optnshowfit')    
    if (*pState).ftype eq 2 then begin  ;MACS
       macsdetmenu = widget_button(optnmenu,value='MACS Detector Choice',/menu,uname='macsdetMenu',/separator)
       void = dm_widget_button(macsdetmenu,value='SPEC',uname='macs_det_spec',set_button=((*pState).y_typ eq 0))
       void = dm_widget_button(macsdetmenu,value='DIFF',uname='macs_det_diff',set_button=((*pState).y_typ eq 1))
    endif                 
             
    ;tools menu
    toolmenu    = widget_button(bar,value='Tools',/menu,uname='toolMenu')  
    calctthBut  = widget_button(toolmenu,value='Calculate 2theta',uname='calctthbut')
    calclattBut = widget_button(toolmenu,value='Calculate Lattice Parameter',uname='calclattbut')
    calca2But   = widget_button(toolmenu,value='Calculate '+(*(*pState).a2name)[(*pState).ftype]+' Reset Value',uname='calca2but')
    calca2Rot   = widget_button(toolmenu,value='Calculate '+(*(*pState).a2name)[(*pState).ftype]+' Rotation Angle',uname='calca2rot')
    e2lambdaBut = widget_button(toolmenu,value='E to Wavelength Conversion',/separator,uname='e2lambdabut')
    if (*pState).ftype eq 2 then $  ;MACS
    kidlimitBut = widget_button(toolmenu,value='Kidney Angle Limit',uname='kidlimitbut')
    
    if !version.os_family eq 'Windows' then $
       mbar_sep = widget_label((*pState).tlb,sensitive=0,value=' ',/dynamic_resize,scr_ysize=4)
    
    tmpbase = widget_base((*pState).tlb,/row)   
         
    left = widget_base(tmpbase,/col)
    (*pState).filesel = obj_new('dm_filesel',parent=left,xsize=160,ysize=20,/frame,filter=(*(*pState).fextn)[(*pState).ftype],path=(*pState).datDir,$
       ncnrftp=file_test((*pState).workDir,/directory,/write),bufferdir=(*pState).workDir)
    if !version.os_family eq 'unix' then $
       center = widget_base(tmpbase,/grid_layout,/col,/align_bottom,xpad=0,ypad=0,space=0) $
    else $
       center = widget_base(tmpbase,/grid_layout,/col,/align_bottom)
    
    void     = widget_label(center,value='Det. Number')
    detList  = dm_widget_droplist(center,value=det_info,uname='detlist')
    void     = widget_label(center,value='Time Channel',uname='timelabel',sensitive=((*pState).ftype eq 0))
    timeList = dm_widget_droplist(center,value=['All','Elastic','Specify'],select=(*pState).t_chan,uname='timelist',sensitive=((*pState).ftype eq 0))
    void     = widget_label(center,value='X-axis')
    xaxisList= dm_widget_droplist(center,value=x_info,uname='xaxislist')
    for i=0,1 do void = widget_label(center,value=' ',sensitive=0)
    loadBut  = widget_button(center,value='Load Data',uname='loadbut')
    fitBut   = widget_button(center,value='Gauss Fit',uname='fitbut')
    saveBut  = widget_button(center,value='Save Plot',uname='savebut')
    dataBut  = widget_button(center,value='Save Data',uname='databut')
    printBut = widget_button(center,value='Print',uname='printbut')
    exitBut  = widget_button(center,value='Exit',uname='exitbut')

    geom  = widget_info(exitBut,/geometry)
    widget_control,detList,xsize=geom.scr_xsize
    widget_control,timeList,xsize=geom.scr_xsize
    widget_control,xaxisList,xsize=geom.scr_xsize

    geom  = widget_info(left,/geometry)
    ysize = geom.scr_ysize
    xsize = fix(1.4*ysize,type=3)
    (*pState).plotWin = obj_new('dm_plot',xsize=xsize,ysize=ysize,widgetbase=tmpbase,group_leader=(*pState).tlb,/compound,background=(*pState).bgcolor,/isolatin1)
    
    dm_center_kid, (*pState).tlb   ;centering the top level base
    widget_control,(*pState).tlb,/realize,/map
    (*pState).plotWin->draw,/view
    (*pState).plotWin->totop
    
    geom  = widget_info((*pState).tlb,/geometry)
    (*pState).geom = [geom.scr_xsize,geom.scr_ysize]
    
    widget_control,(*pState).tlb,set_uvalue=pState

    if xregistered('dave',/noshow) and (n_elements(event) ne 0) then begin
       (*pState).enotebook = 1b
       RET = DAVE_SET_FOCUS((*pState).tlb)
    endif
    
    xmanager,registerName,(*pState).tlb,cleanup='dcs_cryst_align_Exit',/no_block
end