; $Id: dm_check_macskidney.pro,v 1.4 2013/04/25 14:22:09 ymqiu Exp $
;#######################################################################
; 
; NAME:
;  dm_check_macskidney
;
; PURPOSE:
;  check macs kideney angle to see if it is out of range
;  
; CATEGORY:
;  dcs_mslice
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  April, 2013
; 
; HISTORY:
; 02/2013: use NG0 limits
; 04/2013: update with BT9 limits
; 
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

;check kidney position, return the index of good kidney angles
;parameters:
;  kidney:      kidney angles [nkid], software angle
;  Ei:          Ei values [nkid]  
;  count:       return the number of good kidney angles
;keyword:
;  complement:  return the index of bad kidney angles
;  ncomplement: return the number of bad kidney angles
;  limits:      save the limits [ne,3], hardware angle
;  offset:      hardware offset angle, it corresponds to the hardware angle when software angle is 0
;  extra keywords are for interpol routine
function dm_check_macskidney,kidney,Ei,count,complement=complement,ncomplement=ncomplement,limits=limits,offset=offset,_ref_extra=_extra
    n_kid = n_elements(kidney)
    count = n_kid & ncomplement = 0 & complement = -1
    if n_kid eq 0 then return,-1
    if n_elements(Ei) ne n_kid then return,-1
    if n_elements(offset) eq 0 then offset=2.17  ;as measured on 4/24/2013
    limits = [[2.35000,      28.1389,      47.0889],$
              [2.50000,      19.9722,      43.5167],$
              [2.75000,       7.2944,      38.3667],$
              [3.00000,       0.6278,      33.9167],$
              [3.10000,      -1.7500,      32.0611],$
              [3.20000,      -4.2944,      30.2833],$
              [3.25000,      -5.4667,      29.6111],$
              [3.30000,      -6.4944,      28.8611],$
              [3.50000,     -10.3167,      25.6278],$
              [3.75000,     -14.3500,      22.1333],$
              [4.00000,     -17.6333,      18.7444],$
              [4.25000,     -20.4389,      15.7500],$
              [4.50000,     -22.8556,      12.7111],$
              [4.75000,     -25.0389,      10.0722],$
              [5.00000,     -26.9222,       7.6889],$
              [5.25000,     -28.4722,       5.0833],$
              [5.50000,     -29.9667,       2.4944],$
              [5.75000,     -31.2111,       0.1000],$
              [6.00000,     -32.4389,      -2.1333],$
              [6.25000,     -33.4389,      -4.1667],$
              [6.50000,     -34.4833,      -6.0500],$
              [6.75000,     -35.4222,      -7.9722],$
              [7.00000,     -36.0833,      -9.6833],$
              [7.25000,     -36.7889,     -11.0444],$
              [7.50000,     -39.9611,     -12.7722],$
              [7.75000,     -39.8556,     -13.9722],$
              [8.00000,     -39.6500,     -15.6056],$
              [8.25000,     -39.7111,     -16.7944],$
              [8.50000,     -40.1222,     -18.5444],$
              [8.75000,     -40.8556,     -19.4833],$
              [9.00000,     -41.2111,     -20.9722],$
              [9.25000,     -41.7389,     -22.1167],$
              [9.50000,     -42.2611,     -23.2444],$
              [9.75000,     -42.6111,     -24.1000],$
              [10.0000,     -42.9333,     -25.2111],$
              [10.2500,     -43.4056,     -26.4444],$
              [10.5000,     -43.8444,     -27.2278],$
              [10.7500,     -44.2611,     -28.0444],$
              [11.0000,     -44.6000,     -29.2000],$
              [11.2500,     -44.9111,     -30.0167],$
              [11.5000,     -45.1278,     -30.7000],$
              [11.7500,     -45.3722,     -31.4667],$
              [12.0000,     -45.7000,     -32.7500],$
              [12.2500,     -45.9722,     -33.2611],$
              [12.5000,     -46.4389,     -33.8611],$
              [12.7500,     -46.6556,     -34.8000],$
              [13.0000,     -46.8167,     -35.4111],$
              [13.2500,     -46.9500,     -35.9889],$
              [13.5000,     -47.0722,     -36.7278],$
              [13.7500,     -47.3722,     -37.5833],$
              [14.0000,     -47.5278,     -38.0778],$
              [14.2500,     -47.6500,     -38.5222],$
              [14.5000,     -47.9444,     -39.0556],$
              [14.7500,     -48.2778,     -40.0278],$
              [15.0000,     -48.2889,     -40.6389],$
              [15.2500,     -48.6444,     -40.6889],$
              [15.5000,     -48.7056,     -41.1500],$
              [15.7500,     -48.7278,     -41.7722],$
              [16.0000,     -48.8556,     -42.3444]]
    limits = transpose(limits)
    llim   = interpol(limits[*,1],limits[*,0],Ei,_extra=_extra)
    ulim   = interpol(limits[*,2],limits[*,0],Ei,_extra=_extra)
    index  = where((kidney+offset ge llim) and (kidney+offset le ulim),count,complement=complement,ncomplement=ncomplement)
    return,index
end