; $Id: dm_copyftp.pro,v 1.1 2013/03/20 15:18:56 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_copyftp
;
; PURPOSE:
;  this program allows users to copy files from ncnr ftp server to a local directory 
;
; CATEGORY:
;  dcs_mslice file tools
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  March, 2013
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

;clear all objects and widget hierarchy
pro dm_copyftp_Exit,tlb
    widget_control,tlb,get_uvalue=state,/no_copy
    obj_destroy,[state.open_filesel,state.save_filesel]
    widget_control,tlb,/destroy
end

;event handler
pro dm_copyftp_event,event
    compile_opt IDL2    ;,strictarrsubs
                        ;idl2=defint32,strictarr
                        ;strictarr:   [] to index array
                        ;strictarrsubs: error when out-of-range indices,IDL5.6 or after
    WIDGET_CONTROL,/HOURGLASS
    widget_control,event.handler,get_uvalue=state,/no_copy
    state.open_filesel->getproperty,path=open_path,file=open_file,ftpobj=ftpobj
    state.save_filesel->getproperty,path=save_path

    ;catch and ignore all errors in this program
    catch, myerror
    if myerror ne 0 then begin
       ok = dialog_message(dialog_parent=state.tlb,!error_state.msg,/error,/center)
       catch,/cancel
       widget_control,event.handler,set_uvalue=state,/no_copy
       return
    end
    
    case (event.id) of
       state.tlb:   widget_control,event.id,scr_xsize=state.geom[0],scr_ysize=state.geom[1]
       state.exitBut:begin
          widget_control,event.handler,set_uvalue=state,/no_copy
          widget_control,event.handler,/destroy
          return
         end   
       state.copyBut:begin
          n_files = n_elements(open_file)
          if n_files eq 0 then mesg = 'Please select ftp files first.'
          if ~obj_valid(ftpobj) then mesg = 'Please copy files from the NCNR ftp server.'
          if ~file_test(save_path,/directory,/write) then mesg = save_path+' is unwritable.' 
          if n_elements(mesg) ne 0 then begin
             ok = dialog_message(mesg,/error,/center,dialog_parent=event.handler)
             break
          endif
          while strmid(open_path,0,1,/reverse_offset) eq state.pathsep[1] do open_path = strmid(open_path,0,strlen(open_path)-1)
          while strmid(save_path,0,1,/reverse_offset) eq state.pathsep[0] do save_path = strmid(save_path,0,strlen(save_path)-1)
          for i=0L,n_files-1 do begin
              if i eq 0 then $
                 mesg = obj_new('dm_progress',title='Copying...',message='Copying '+open_file[i],group_leader=state.tlb) $
              else begin
                 if (~ obj_valid(mesg)) then break
                 mesg->update,message='Copying '+open_file[i],title='Copying...  '+dm_to_string(i+1)+'/'+dm_to_string(n_files)
              endelse
              ok = ftpobj->GetFileContent(open_path+state.pathsep[1]+open_file[i],localfilename=save_path+state.pathsep[0]+open_file[i])
              state.save_filesel->set_path      ;update the save directory
          endfor
          state.open_filesel->set_path,open_path,/ftp
          state.open_filesel->clear_file,fileloc=open_file[0]
          if obj_valid(mesg) then obj_destroy,mesg
         end  
       else:
    endcase
    widget_control,event.handler,set_uvalue=state,/no_copy
end

;main program
pro dm_copyftp,event,workDir=workDir,filter=filter
    if n_elements(workDir) eq 0 then workDir = sourcepath()
    if ~file_test(workDir[0],/directory,/write) then begin
       ok = dialog_message('Please specify a writable working directory.',/error,/center,dialog_parent=(n_elements(event) ne 0?event.top:0))
       return
    endif
    state={ group_leader:    0L, $   ;group leader
         tlb:                0L, $   ;top level base
         open_filesel: obj_new(),$   ;open file selector
         save_filesel: obj_new(),$   ;save file selector
         copyBut:            0L, $   ;copy button
         exitBut:            0L, $   ;exit button
         tooltip:            '', $   ;tool tip for copy button
         pathsep:       ['',''], $   ;path separator 
         geom:    [0L,0L,0L,0L]  $   ;xsize,ysize,xoffset,yoffset
    }
    registerName    ='dm_copyftp'
    if xregistered(registerName) then return   ;only allow one copy to be running at one time
    
    ;get the correct path separator
    if float(!version.release) ge 5.5 then pathsep=call_function("path_sep") $
    else begin
       defsysv,'!delimiter',exists=exists
       if (~ exists) then begin
          case !version.os_family of
            'Windows':  defsysv,'!delimiter','\'
            'unix':     defsysv,'!delimiter','/'
            'MacOS':    defsysv,'!delimiter',':'
            else:       defsysv,'!delimiter','/'
          endcase
       endif
       pathsep = !delimiter
    endelse
    state.pathsep = [pathsep,'/']
    
    if n_elements(event) ne 0 then begin
       state.group_leader = event.top
       state.tlb = widget_base(title='Copy files from NCNR ftp server',/col,kill_notify='dm_copyftp_Exit',group_leader=event.top,/tlb_size_event,mbar=mbar,xpad=0,ypad=0)
    endif else $
       state.tlb = widget_base(title='Copy files from NCNR ftp server',/col,kill_notify='dm_copyftp_Exit',/tlb_size_event,mbar=mbar,xpad=0,ypad=0)
    
    ;menu bar
    filemenu = widget_button(mbar,value='File',/menu)
    state.exitBut = widget_button(filemenu,value='Exit')
     
    ;menu bar separator for windows system
    if !version.os_family eq 'Windows' then begin
       mbar_sep    = widget_label(state.tlb,sensitive=0,/dynamic_resize,value=' ',scr_ysize=5)
       tmp         = 0
    endif else tmp = 4
    
    row            = widget_base(state.tlb,/row,ypad=0)
    openCol        = widget_base(row,/col,ypad=0)
    cmdCol         = widget_base(row,/col,/align_center,ypad=0,xpad=0)
    saveCol        = widget_base(row,/col,ypad=0)

    label          = widget_label(openCol,value='NCNR ftp server')
    label          = widget_label(saveCol,value='local save directory')
    
    if n_elements(filter) eq 0 then filter = '.*'

    state.open_filesel = obj_new('dm_filesel',parent=openCol,ysize=18+tmp,/frame,filter=filter,path='ftp://ncnr.nist.gov',/ncnrftp,group_leader=state.tlb,bufferdir=workDir)
    state.save_filesel = obj_new('dm_filesel',parent=saveCol,ysize=18+tmp,/frame,filter=filter,path=workDir)
    state.copyBut  = widget_button(cmdCol,value='  Copy->  ')
    state.tooltip  = 'Choose the local directory for the files to be saved in, select the files from the ftp server, then click this button to copy files. '
    if float(!version.release) ge 5.6 then $
       widget_control,state.copyBut,tooltip=state.tooltip

    dm_center_kid, state.tlb   ;centering the top level base
    widget_control,state.tlb,/realize
    geom = widget_info(state.tlb,/geometry)
    state.geom = [geom.scr_xsize,geom.scr_ysize,geom.xoffset,geom.yoffset]
    tlb  = state.tlb
    widget_control,state.tlb,set_uvalue=state,/no_copy
    xmanager,registerName,tlb,cleanup='dm_copyftp_Exit',/no_block
end