; $Id: dm_q2hkl.pro,v 1.4 2013/03/15 21:57:46 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_q2hkl
;
; PURPOSE:
;  project Q[3,ne,ndet] onto three vectors abc_r[3,3]
;
; CATEGORY:
;  dcs_mslice
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  February, 2013
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

;keyword:
;  n_fold: symmetry rotation fold, if present, alwasy rotate to the first quardant of the lattice coordinate system in the scattering plane
function dm_q2hkl,Q,abc_r,n_fold=n_fold,debug=debug
    if keyword_set(debug) then begin
       print,'%%Q to hkl...'
       current=systime(/sec)
    endif
    if n_elements(n_fold) ne 0 then fold = (1>(fix(n_fold))) else fold = 1  
    ABC   = dm_basis_hkl(abc_r)
    Qsize = size(Q,/dimension)
    nQdim = n_elements(Qsize)
    case nQdim of
       2:    tmp = Q                                         ;Q=[3,ndet] diffuse, 1 file, single E
       3:    tmp = reform(Q,3,Qsize[1]*Qsize[2])             ;Q=[3,nen,ndet] or Q=[3,ndet,nfile]
       4:    tmp = reform(Q,3,Qsize[1]*Qsize[2]*Qsize[3])    ;Q=[3,nen,ndet,nfile]
       else: tmp = Q
    endcase
    ABC = transpose(ABC)
    hkl = tmp##ABC
    if fold gt 1 then begin
       index = where((hkl[0,*] lt 0) or (hkl[1,*] lt 0),count)
       nrot  = 0
       sinpsi = sin(2*!pi/fold)
       cospsi = cos(2*!pi/fold)
       R = [[cospsi,-sinpsi,0.0],[sinpsi,cospsi,0.0],[0.0,0.0,1.0]]
       while (count gt 0) and (nrot lt fold) do begin
           tmp[*,index] = tmp[*,index]##R
           hkl[*,index] = tmp[*,index]##ABC
           index = where((hkl[0,*] lt 0) or (hkl[1,*] lt 0),count)
           nrot = nrot+1
       endwhile
    endif
    case nQdim of
       3:    hkl = reform(hkl,3,Qsize[1],Qsize[2])
       4:    hkl = reform(hkl,3,Qsize[1],Qsize[2],Qsize[3])
       else:
    endcase
    if keyword_set(debug) then begin
       print,'%%Q to hkl finished in ',systime(/sec)-current,' sec.'
    endif
    return,hkl
end